
//--- PdfDoc.hpp
//--- Copyright Thilo Brai, 2012
//--- Creative Commons Lizenz: CC-by-sa
//
//--- PdfDoc.hpp ist der eigentliche Pdf-Kreator.
//--- Gesteuert wird er von PdfGraphic. Eine Klasse die 
//--- sich drum kmmert, dass alle bentigten Pdf-Objekte
//--- erstellt und in PdfDoc verlinkt werden.
//
//--- Hier werden die ganzen Pdf-Objekte verwaltet.
//--- Der Einfachheit halber genau so, wie sie spter 
//--- auch in der Pdf-Datei landen.
//--- Die einzigen Funktionen, die public sein mssten
//--- sind init (Blattbreite, Blatthoehe), createPdf ()
//--- und cPdfDoc::Info->addKey (Schluessel, Wert).
//--- Im Info-Dictionary stehen Infos zu Title-,
//--- Subject-, Author- etc. Infos fr die Pdf-Datei (siehe 
//--- Ende des zweiten Beispiels).
//--- Der Rest der Pdf-Logistik wird von der Klasse PdfGraphic
//--- gesteuert.

#ifndef PdfDoc_hpp
#define PdfDoc_hpp

#include "PdfObj.hpp"


//--- die Objects werden hier gespeichert und nummeriert
class cPdfDoc {
public:
	cPdfDoc () {
		MediaBox.set ("[0 0 595 842]");	// DIN A4
		objects   = new cArray (-1); //= new Array ();	// speichert alle obj-Elemente
		trailer   = 0;// trailer sitzt auf Spezialposition 0
		Catalog   = 0;// /Pages, /Outlines ...
		Pages     = 0;// /Count, /Kids [n 0 R]
		Info      = 0;		//Title, Author ...
		arPages   = new cArray (-1);
		fonts     = new cArray (-1);
		bookmarks = new cArray (-1); // lebt hier, wird von PdfGraphic gefllt.
	}

	~cPdfDoc () {
		delete objects;
		delete arPages;
		delete fonts;
		delete bookmarks;
	}

	//--- Die bentigten Pdf-Objekte --------
	cPdfString MediaBox;		// = "[0 0 595 842]";	// DIN A4
	cArray    *objects;		// = new Array ();	// speichert alle obj-Elemente
	int        objCounter;	// = 0;
	cPdfDict  *trailer;		// = 0;// trailer sitzt auf Spezialposition 0
	cPdfDict  *Catalog;		// = 0;// /Pages, /Outlines ...
	cPdfDict  *Pages;			// = 0;// /Count, /Kids [n 0 R]
	cPdfDict  *Info;			// = 0;// Title, Author ...
	cArray    *arPages;		// = new Array ();
	cArray    *fonts;			// = new Array ();
	cArray    *bookmarks;	// = new Array ();// lebt hier, wird von PdfGraphic gefllt.

	void init (int width = -1, int height = -1) {
		if (width < 0)
			width = 595;		// DIN A4
		else if (width < 72)
			width = 72;
		else if (width > 3240)
			width = 3240;
		if (height < 0) {
			switch (width) {
			case  420: case  421: case  422: height =  595; break;	// DIN A5
			case  594: case  595: case  596: height =  842; break;	// DIN A4
			case  841: case  842: case  843: height = 1190; break;	// DIN A3
			case 1189: case 1190: case 1191: height = 1684; break;	// DIN A2
			case 1683: case 1684: case 1685: height = 2380; break;	// DIN A1
			default: height = width * 842 / 595;
			}
		}
		if (height < 72)
			height = 72;
		else if (height > 3240)
			height = 3240;
		objCounter = 0;
		cAString temp;
		temp.Format ("[0 0 %d %d]", width, height);
		MediaBox.set (temp);
		trailer = newPdfDictObj ("/trailer");	// objCounter == 0;
		Catalog = newPdfDictObj ("/Catalog");	// objCounter == 1;
		Pages   = newPdfDictObj ("/Pages");		// objCounter == 2;
		Info    = newPdfDictObj ("/Info");
		Info->addKey ("/Producer", "(C++ PDF-Kreator)");
		Info->addKey ("/CreationDate", PDFdate ().Get ());
		//--- eine Seite erzeugen
		getPage (0);
	}

	void cleanup () {	//--- So. Alles erledigt. Die Objekte aufrumen.
		delete objects;		objects   = new cArray (-1);
		/*delete trailer;*/	trailer   = 0;
		/*delete Catalog;*/	Catalog   = 0;
		/*delete Pages;*/		Pages     = 0;
		/*delete Info;*/		Info      = 0;
		delete arPages;		arPages   = new cArray (-1);
		delete fonts;			fonts     = new cArray (-1);
		delete bookmarks;		bookmarks = new cArray (-1);
	}

	//--- neue Seite oder bestehende Seite holen
	cPdfDict *getPage (int pageNum) {
		cAString temp;
		while (arPages->getLength () < pageNum + 1) {
			cPdfDict *page = newPdfDictObj ("/Page");
			arPages->add (page);
			page->addKey ("/MediaBox", MediaBox);
			page->addKey ("/Parent", asRObj (Pages).Get ());
			page->addKey ("/Contents", asRObj (newPdfStreamObj ()).Get ());
			page->addKey ("/Resources", asRObj (newPdfDictObj ("/Resources")).Get ());	// Eine wirklich blde Idee von Adobe, die PostSript-ProcSets-Anforderungen in ein Archivformat zu integrieren...
		}
		return (cPdfDict *)arPages->get (pageNum);
	}

	cPdfDict *newPdfDictObj (const char *Type)
	{
		cPdfDict *obj = new cPdfDict (objCounter++, Type);
		objects->add (obj);
		return obj;
	}

	cPdfStream *newPdfStreamObj ()
	{
		cPdfStream *obj = new cPdfStream (objCounter++);
		objects->add (obj);
		return obj;
	}

	cArray *newPdfArrayObj ()
	{
		cArray *obj = new cArray (objCounter++);
		objects->add (obj);
		return obj;
	}

	/*cObject *getRecentPdfObj (const char *Type)
	{
		int i = objects.getLength () - 1;
		while (i >= 0) {
			if (objects [i].getType () == Type)
				return objects [i];
			i--;
		}
		return 0;
	}*/

	cAString asRObj (cObject *obj)
	{
		cAString s;
		s.Format ("%d 0 R", obj->getObjNum ());
		return s;
	}

	cObject *asObj (const char *relObj)
	{
		int i = atoi (relObj);
		return objects->get (i);
	}

	cObject *asObj (cObject *relObj)
	{
		if (relObj->getType () == ObjString)
			return asObj (((cPdfString *)relObj)->string);
		return 0;
	}

	cPdfDict *registerFont (const char *schriftname, const char *type, const char *encoding)
	{
		cPdfDict *obj = findFont (schriftname);
		if (!obj) {
			cAString fontName;
			fontName.Set (schriftname, 5);
			fontName.FormatAdd ("%d", fonts->getLength ());
			obj = newPdfDictObj ("/Font");
			obj->addKey ("/BaseFont", schriftname);
			obj->addKey ("/Name", fontName);
			obj->addKey ("/Subtype", type);
			obj->addKey ("/Encoding", encoding);
			fonts->add (obj);
		}
		return obj;
	}

	cPdfDict *findFont (const char *schriftname)
	{
		int i, length = fonts->getLength ();
		for (i = 0; i < length; i++) {
			cPdfDict *font = (cPdfDict *)fonts->get (i);
			if (font) {
				cPdfString *name = (cPdfString *)font->getValueOfKey ("/BaseFont");
				if (name) {
					if (0 == strcmp (name->string, schriftname))
						return font;
				}
			}
		}
		return 0;
	}

	void bookmarksMake ()
	{
		//--- 0 = title, 1 = level, 2 = open/Count, (2 = gotoPage), 3 = Dest/A-Key , 4 = bkm
		cAString temp, t2;
		cPdfDict *Outlines = newPdfDictObj ("/Outlines");
		Catalog->addKey ("/Outlines", asRObj (Outlines).Get ());
		if (!Catalog->getValueOfKey ("/PageMode"))
			Catalog->addKey ("/PageMode", "/UseOutlines");

		int i, recentLevel, level = -1;
		cPdfDict *bkm;
		cPdfDict *parentBkm = Outlines;
		cPdfDict *prevBkm = Outlines;
		cPdfDict *firstBkm = 0;

		for (i = bookmarks->getLength () - 1; i >= 0; i--)
			bookmarkCountVisibleChilds (i);
		Outlines->addKey ("/Count", bookmarkCountVisibleChilds (-1));

		for (i = 0; i < bookmarks->getLength (); i++) {
			recentLevel = level;
			cBookmark *ar = (cBookmark *)bookmarks->get (i);	// [Title, Level, open /Dest...]
			bkm = newPdfDictObj ("/Outline");
			ar->prev = bkm;	// so finden wir den Vater, Vorgnger leichter
			bkm->addKey ("/Title", temp.Format ("(%s)"
				, EscapePsText (Win2PdfEncoding (ar->Title.Get ()).Get ())).Get ());
			//--- /Dest... den ersten Key holen
			int j;
			cAString key, string;
			const char *s = ar->Dest.Get ();
			int len = strlen (s);
			for (j = 1; (j < len) && (s [j] > ' '); j++);
			key.Set (s, 0, j);
			for (; (j < len) && (s [j] <= ' '); j++);
			bkm->addKey (key, s + j);

			if (ar->open != 0)	// positive sind sichtbar, negative sind eingeklappt.
				bkm->addKey ("/Count", ar->open);
			if (ar->Level > level) {	// Level
				level++;
				parentBkm = prevBkm;
				bkm->addKey ("/Parent", asRObj (parentBkm));
				firstBkm = bkm;
				parentBkm->addKey ("/First", asRObj (bkm));	// schreibt auch in /Outlines
				parentBkm->addKey ("/Last", asRObj (bkm));	// wird evtl. vom nchsten berschrieben
			} else if (ar->Level == level) {
				bkm->addKey ("/Parent", asRObj (parentBkm));
				prevBkm->addKey ("/Next", asRObj (bkm));
				bkm->addKey ("/Prev", asRObj (prevBkm));
				parentBkm->addKey ("/Last", asRObj (bkm));	// wird evtl. vom nchsten berschrieben
			} else if (ar->Level < level) {
				level = ar->Level;
				//--- Vorgnger und Vater suchen
				for (int ii = i - 1; ii >= 0; ii--) {
					if (((cBookmark *)bookmarks->get (ii))->Level == level) {
						prevBkm = ((cBookmark *)bookmarks->get (ii))->prev;
						parentBkm = (cPdfDict *)asObj (prevBkm->getValueOfKey ("/Parent"));
						break;
					}
				}
				bkm->addKey ("/Parent", asRObj (parentBkm));
				bkm->addKey ("/Prev", asRObj (prevBkm));
				prevBkm->addKey ("/Next", asRObj (bkm));
				parentBkm->addKey ("/Last", asRObj (bkm));
			}
			prevBkm = bkm;
		}
	}

	void solvePageX (cObject *obj)
	{
		if (obj->getType () != ObjString)
			return;

		const char *string = ((cPdfString *)obj)->string.Get ();
		cAString temp, rel;

		const char *fi = strstr (string, "{Page");
		while (fi) {
			int i;
			for (i = 0; isdigit (fi [5 + i]); i++);
			if (i && (fi [5 + i] == '}')) {
				int page = atoi (fi + 5);
				rel = asRObj ((page < arPages->getLength ())
									? arPages->get (page)
									: arPages->get (0));
				cAString s1;
				s1.Set (string, fi - string);
				s1 += rel;
				temp += s1;
				string = fi + 5 + 1 + i;
			}
			//--- Falls noch mehr {PageX} da sind.
			fi = strstr (fi + 5, "{Page");
		}
		if (temp.Length ()) {
			temp += string;
			((cPdfString *)obj)->set (temp);
		}
	}

	//--- Schaut im bookmarks-Array nach und verlsst sich darauf,
	//--- dass verschachtelte Zweige schon richtig gezhlt sind.
	int bookmarkCountVisibleChilds (int index)
	{
		int i, level, count = 0;
		cBookmark *ar, temp;

		if (index < -1) {
			return 0;
		} else if (index == -1) {
			ar = &temp;
			//ar->Title = "";
			ar->Level = -1;
			ar->open = 1;
			//ar->Dest = "";
		} else if (index < bookmarks->getLength ()) {
			ar = (cBookmark *)bookmarks->get (index);
		} else
			return 0;

		level = ar->Level;

		for (i = index + 1
			; (i < bookmarks->getLength ())
				&& (((cBookmark *)bookmarks->get (i))->Level == level + 1)
			; i++)
		{
			count++;
			cBookmark *t = (cBookmark *)bookmarks->get (i);
			if (t->open > 0)
				count += t->open;
		}
		if (count) {	// Wenn es Untergeordnete gibt...
			if (ar->open > 0)
				ar->open = count;
			else
				ar->open = -count;
		} else
			ar->open = 0;

		return ar->open;
	}

	//--- Die Objects-Daten nur noch in der richtigen Reihenfolge ausgeben.
	const char *createPdf (cAString &pdf)
	{
		int i;
		cAString temp;

		//--- Die Seiten von arPages in /Pages eintragen
		Catalog->addKey ("/Pages", asRObj (Pages).Get ());
		cAString sKids = "[";
		for (i = 0; i < arPages->getLength (); i++) {
			sKids += asRObj (arPages->get (i));
			sKids += " ";
		}
		sKids += "]";
		Pages->addKey ("/Kids", sKids);
		Pages->addKey ("/Count", arPages->getLength ());

		//--- Die Bookmarks erzeugen
		if (bookmarks->getLength ())
			bookmarksMake ();

		//--- Die {PageX}-Variablen auflsen
		for (i = 1; i < objects->getLength (); i++) {
			if (objects->get (i)->getType () == ObjDict) {	// ein Dictionary
				int dictLen = ((cPdfDict *)objects->get (i))->getLength ();
				for (int iDict = 0; iDict < dictLen; iDict++) {
					cObject *value = ((cPdfDict *)objects->get (i))->getValueOfIndex (iDict);
					solvePageX (value);
				}
			}
		}

		//--- So, jetzt aber...
		pdf = "%PDF-1.2";
		pdf += NL;
		pdf += NL;

		int startxref;
		tCollection <int> xref;
		xref.Alloc (objCounter);
		xref.Add (0);

		for (i = 1; i < objects->getLength (); i++) {
			cAString t2;
			xref.Add (pdf.Length ());
			temp.Format ("%d 0 obj%s", objects->get (i)->getObjNum (), NL);
			temp += objects->get (i)->getObjString (t2);
			temp.FormatAdd ("%sendobj%s%s", NL, NL, NL);
			pdf += temp;
		}

		//--- Der xref-Abschnitt
		startxref = pdf.Length ();
		pdf.FormatAdd ("xref%s"
							"0 %d%s"
							"0000000000 65535 f%s"
								, NL
								, objects->getLength (), NL
								, NL);

		for (i = 1; i < objects->getLength (); i++) {
			pdf.FormatAdd ("%010d 00000 n%s", *xref.GetItAsPtr (i), NL);
			//var l = "0000000000" + xref [i];
			//pdf += l.substr (l.length - 10, 10) + " 00000 n" + NL;
		}

		//--- Der trailer-Abschnitt
		trailer->addKey ("/Size", objects->getLength ());
		trailer->addKey ("/Root", asRObj (Catalog));
		trailer->addKey ("/Info", asRObj (Info));

		pdf.FormatAdd ("trailer%s"
							"%s%s"
								, NL
								, trailer->getObjString (temp), NL);

		//--- Der startxref-Abschnitt
		pdf.FormatAdd ("startxref%s"
							"%d%s"
							"%%%%EOF"
								, NL
								, startxref, NL);

		if (1)
			cleanup ();

		return pdf;
	}
};

#endif //PdfDoc_hpp
