
//--- PdfGraphic.hpp
//--- Copyright Thilo Brai, 2012
//--- Creative Commons Lizenz: CC-by-sa
//
//--- Kmmert sich um die PdfDoc-Verwaltung.
//--- Stellt einige Grafik-Funktionen zur Verfgung.
//--- Merkt sich Farben-, Fonts-Einstellungen und
//--- schreibt sie zur richtigen Zeit in den Pdf-Stream.
//--- (Ein bisschen Pdf-Stream-Basic sollte man aber haben.)
//
//--- Im Sourcetext reicht es, wenn man PdfGraphic.hpp inkludiert.
//--- Beispiele:
//--- #include "PdfDoc.hpp"

#ifndef PdfGraphic_hpp
#define PdfGraphic_hpp

#include "PdfDoc.hpp"

extern void alert (const char *);
extern cAString EscapePsText (const char *text);

//--- Mit cPdfGraphic werden die Pdf-Streams erzeugt.
//--- Die Zeigen-/Text-Funktionen abstrahieren etwas,
//--- damit man auch ohne intime Pdf-Kenntnisse
//--- gltige Pdf-Streams produzieren kann.
//--- Mit cPdfGraphic::Add (string) kann man den Pdf-Text 
//--- auch direkt schreiben. Demo dazu in PdfWTest.cpp.
class cPdfGraphic { // (pdf)
private:
	cPdfGraphic () { return; }
public:
	cPdfGraphic (cPdfDoc &pdf) { Pdf = &pdf; }
	~cPdfGraphic () { return; }
public:
	cPdfDoc    *Pdf;			// = pdf;	// ohne die Pdf geht nix
	cPdfDict   *Page;			// = 0;	// ein Dict in Pages
	int         PageIndex;	// = -1;	// aktuell mit InitPage geffnete Seite
	cPdfStream *Contents;	// = 0;	// ein Stream in Pages
	cPdfDict   *Resources;	// = 0;	// ein Dict in Pages

	int         writeAtOnce;// = 1;	// Die Grafikfunktionen schreiben sofort in Contents
									// bei 0 geben sie die Zeichenkette zurck.
	cAString    path;			// = "";

	//--- GraphicState wie in InitPage
	cPdfDict   *gsFont;				// = 0;	// ein Dict in Pdf
	cAString    gsFontName;			// = "";	// /Helv0, abhngig von Schriftname
	double      gsFontSize;			// = 10;
	int         gsFontChanged;		// = 1;
	int         gsTextRenderingMode;	// = 0;
	int         gsTextRenderingModeChanged;	// = 1;

	cAString    gsFillColor;		// = "0 0 0";
	int         gsFillColorChanged;	// = 1;

	cAString    gsStrokeColor;		// = "0 0 0";
	int         gsStrokeColorChanged;	// = 1;
	double      gsLineWidth;		// = .25;
	int         gsLineWidthChanged;	// = 1;
	int         gsLineCap;			// = 1;	// 0 = endet eckig kurz, 1 = rund, 2 = eckig lang
	int         gsLineCapChanged;	// = 1;
	int         gsLineJoin;			// = 1;	// 0 = Ecke spitz, 1 = rund, 2 = abgeflacht
	int         gsLineJoinChanged;// = 1;
	cAString    gsLineDash;			// = "[]0";
	int         gsLineDashChanged;// = 1;
public:
	cPdfDict *InitPage (int pageIndex)	// Zhlung beginnt bei 0
	{
		if (Pdf && Pdf->objCounter) {
			//--- page holen und dann die /Resources setzen
			Page = Pdf->getPage (pageIndex);	// die erste Seite initieren
			PageIndex = pageIndex;
			Resources = (cPdfDict *)Pdf->asObj (Page->getValueOfKey ("/Resources"));
			Contents = (cPdfStream *)Pdf->asObj (Page->getValueOfKey ("/Contents"));
			//--- Ist nur fr PS-Ausdrucke interessant. Wir geizen nicht und untersttzen alles :)
			cAString temp;
			Resources->addKey ("/ProcSet", "[/PDF /Text /ImageB /ImageC /ImageI]");
			if (!Resources->getValueOfKey ("/Font"))
				Resources->addKey ("/Font"
								, temp.Format ("<<%s"
								//+ "/Helv "+ Pdf.asRObj (helvetica) + NL
								//+ "/Arial "+ Pdf.asRObj (arial) + NL
								">>", NL).Get ());

			gsFont = 0;	// ein Dict in Pdf		// self weg 22.3.2009
			gsFontSize = 10;
			gsFontChanged = 1;
			gsTextRenderingMode = 0;
			gsTextRenderingModeChanged = 1;

			gsFillColor = "0 0 0";
			gsFillColorChanged = 1;

			gsStrokeColor = "0 0 0";
			gsStrokeColorChanged = 1;
			gsLineWidth = .25;
			gsLineWidthChanged = 1;
			gsLineCap = 1;	// 0 = endet eckig kurz, 1 = rund, 2 = eckig lang
			gsLineCapChanged = 1;
			gsLineJoin = 1;	// 0 = Ecke spitz, 1 = rund, 2 = abgeflacht
			gsLineJoinChanged = 1;
			gsLineDash = "[]0";
			gsLineDashChanged = 1;

			return Page;
		} else {
			cAString temp;
			alert (temp.Format ("Fehler!%s"
									"PdfGraphic wurde nicht richtig initialisiert!"
									, NL
									// NL
									// "Pdf = " + Pdf + NL
									// "Pdf.objCounter = " + Pdf.objCounter
									).Get ());
		}
		return 0;
	}

	int Add (const char *s)	// PdfStream erweitern
	{
		//ReportLF ("Add: " + s);
		if (s && *s && Contents && Contents->objNum) {
			Contents->add (s);
			return 1;
		}
		return 0;
	}

	const char *optAdd (const char *s)
	{
		//ReportLF ("optAdd: " + self.writeAtOnce + " " + s);
		if (writeAtOnce) {
			if (s && *s)
				Add (s);
			return "";
		}
		return s;
	}

	//--- Pdf.registerFont ("/Helvetica", "/Type1"   , "/WinAnsiEncoding");
	//--- Pdf.registerFont ("/Arial",     "/TrueType", "/WinAnsiEncoding");
	cPdfDict *SetFont (const char *FontName, const char *Type = 0, const char *Encoding = 0)
	{
		if (Pdf && Pdf->objCounter) {
			cPdfDict *oldFont = gsFont;
			if (!FontName || !*FontName)
				FontName = "/Helvetica";
			if (!Type || !*Type)
				Type = "/TrueType";
			if (!Encoding || !*Encoding)
				Encoding = "/WinAnsiEncoding";
			gsFont = Pdf->registerFont (FontName, Type, Encoding);	// FontDict
			gsFontName = ((cPdfString *)gsFont->getValueOfKey ("/Name"))->string;
			//--- /Font wurde bei InitPage () in /Resources als String gespeichert.
			cAString temp, t2;
			cAString font = ((cPdfString *)Resources->getValueOfKey ("/Font"))->string;
			const char *fontName = strstr (font, temp.Format ("%s ", gsFontName).Get ());
			if (!fontName) {
				font.InsertString (font.Length () - 2
					, temp.Format ("%s %s%s"
						, gsFontName.Get ()
						, Pdf->asRObj (gsFont).Get ()
						, NL).Get ());
				Resources->addKey ("/Font", font);
				gsFontChanged = 1;
			}
			if (oldFont != gsFont)
				gsFontChanged = 1;
		}
		return gsFont;
	}

	void SetFontSize (double s)
	{
		if (gsFontSize != s) {
			gsFontSize = s;
			gsFontChanged = 1;
		}
	}

	void SetTextRenderingMode (int n)
	{
		if (gsTextRenderingMode != n) {
			gsTextRenderingMode = n;
			gsTextRenderingModeChanged = 1;
		}
	}

	// SetFillColor ("0.1 0.5 1")
	void SetFillColor (const char *s)
	{
		if (gsFillColor != s) {
			gsFillColor = s;
			gsFillColorChanged = 1;
		}
	}

	// SetFillColor (0.1, 0.5, 1)
	void SetFillColor (double r, double g, double b)
	{
		cAString neu;
		neu.Format ("%s %s %s"
					, roundmax (r, 2).Get ()
					, roundmax (g, 2).Get ()
					, roundmax (b, 2).Get ());
		SetFillColor (neu);
	}

	// SetStrokeColor ("0.1 0.5 1")
	void SetStrokeColor (const char *s)
	{
		if (gsStrokeColor != s) {
			gsStrokeColor = s;
			gsStrokeColorChanged = 1;
		}
	}

	// SetStrokeColor (0.1, 0.5, 1)
	void SetStrokeColor (double r, double g, double b)
	{
		cAString neu;
		neu.Format ("%s %s %s"
					, roundmax (r, 2).Get ()
					, roundmax (g, 2).Get ()
					, roundmax (b, 2).Get ());
		SetStrokeColor (neu);
	}

	cAString TextOut (double x, double y, const char *text)
	{
		cAString s;

		if (!gsFont && !SetFont ("/Courier", "/Type1"))
			return s;

		if (gsFontChanged) {
			s.FormatAdd ("%s %s Tf%s", gsFontName.Get (), roundmax (gsFontSize, 2).Get (), NL);
			gsFontChanged = 0;
		}
		if (gsTextRenderingModeChanged) {
			s.FormatAdd ("%d Tr%s", gsTextRenderingMode, NL);
			gsTextRenderingModeChanged = 0;
		}
		if ((gsTextRenderingMode == 0) || (gsTextRenderingMode == 2))
			s += checkFillStyle ();
		if ((gsTextRenderingMode == 1) || (gsTextRenderingMode == 2))
			s += checkLineStyles ();

		s.FormatAdd ("1 0 0 1 %s %s Tm (%s) Tj"
							, roundmax (x, 2).Get ()
							, roundmax (y, 2).Get ()
							, EscapePsText (text).Get ());
		s = optAdd (s);
		return s;
	}

	cAString TextOutBlock (double x, double y, const char *text)
	{
		cAString s;
		int writeAtOnceSaved = writeAtOnce;
		writeAtOnce = 0;
		s.Format ("BT%s"
					"%s%s"
					"ET"
					, NL
					, TextOut (x, y, text).Get (), NL);
		writeAtOnce = writeAtOnceSaved;
		s = optAdd (s);
		return s;
	}

	//--- Textpositionierung
	// a b c d x y Tm: setzt die Textmatrix (TM) und die TextLineMatrix (TLM)
	// x y Td: bewegt die Matrizen Tm = TLM = [1 0 0 1 x y] x TLM
	// x y TD  ~  y TL  x y Td
	// T*  ~  0 y Td: bewegt die TM und TLM zur nchsten Zeile
	cAString BeginTextBlock ()
	{
		return optAdd ("BT");
	}

	cAString EndTextBlock ()
	{
		return optAdd ("ET");
	}

	//--- Alle MoveTo, LineTo und CurveTo in path sammeln, weil wir 
	//--- bei PDF nicht mit LineStyles in den Pfad schreiben drfen.
	//--- Die Leerzeichen mssen exakt stimmen, weil damit das Array
	//--- erzeugt wird.
	void MoveTo (double x, double y)
	{
		path.FormatAdd ("m %s %s "
							, roundmax (x, 2).Get ()
							, roundmax (y, 2).Get ());
	}

	void LineTo (double x, double y)
	{
		path.FormatAdd ("l %s %s "
								, roundmax (x, 2).Get ()
								, roundmax (y, 2).Get ());
	}

	void CurveTo (double x1, double y1, double x2, double y2, double x3, double y3)
	{
		path.FormatAdd ("c %s %s %s %s %s %s "
							, roundmax (x1, 2).Get (), roundmax (y1, 2).Get ()
							, roundmax (x2, 2).Get (), roundmax (y2, 2).Get ()
							, roundmax (x3, 2).Get (), roundmax (y3, 2).Get ());
	}

	void ClosePath ()
	{
		path += "h ";
	}

	void Line (double x1, double y1, double x2, double y2)
	{
		MoveTo (x1, y1);
		LineTo (x2, y2);
	}

	void Rect (double x1, double y1, double x2, double y2)
	{
		MoveTo (x1, y1);
		LineTo (x2, y1);
		LineTo (x2, y2);
		LineTo (x1, y2);
		LineTo (x1, y1);
	}

	void RectWH (double x1, double y1, double w, double h)
	{
		MoveTo (x1    , y1    );
		LineTo (x1 + w, y1    );
		LineTo (x1 + w, y1 - h);
		LineTo (x1    , y1 - h);
		LineTo (x1    , y1);
	}

	void Ellipse (double x, double y, double rx, double ry)
	{
		rx = fabs (rx);
		ry = fabs (ry);
		double r5x = rx * .5522852;	// 0,5522852
		double r5y = ry * .5522852;
		path.FormatAdd (	"m %s %s "
								"c %s %s %s %s %s %s "
								"c %s %s %s %s %s %s "
								"c %s %s %s %s %s %s "
								"c %s %s %s %s %s %s "
								, roundmax (x +  rx, 2).Get (), roundmax (y +   0, 2).Get ()	// m

								, roundmax (x +  rx, 2).Get (), roundmax (y - r5y, 2).Get ()
								, roundmax (x + r5x, 2).Get (), roundmax (y -  ry, 2).Get ()
								, roundmax (x +   0, 2).Get (), roundmax (y -  ry, 2).Get ()	// c

								, roundmax (x - r5x, 2).Get (), roundmax (y -  ry, 2).Get ()
								, roundmax (x -  rx, 2).Get (), roundmax (y - r5y, 2).Get ()
								, roundmax (x -  rx, 2).Get (), roundmax (y +   0, 2).Get ()	// c

								, roundmax (x -  rx, 2).Get (), roundmax (y + r5y, 2).Get ()
								, roundmax (x - r5x, 2).Get (), roundmax (y +  ry, 2).Get ()
								, roundmax (x +   0, 2).Get (), roundmax (y +  ry, 2).Get ()	// c

								, roundmax (x + r5x, 2).Get (), roundmax (y +  ry, 2).Get ()
								, roundmax (x +  rx, 2).Get (), roundmax (y + r5y, 2).Get ()
								, roundmax (x +  rx, 2).Get (), roundmax (y +   0, 2).Get ()	// c
								);
	}

	void SetLineCap (int n)
	{
		if (gsLineCap != n) {
			gsLineCap = n;
			gsLineCapChanged = 1;
		}
	}

	void SetLineWidth (double n)
	{
		if (gsLineWidth != n) {
			gsLineWidth = n;
			gsLineWidthChanged = 1;
		}
	}

	void SetLineJoin  (int n)
	{
		if (gsLineJoin != n) {
			gsLineJoin = n;
			gsLineJoinChanged = 1;
		}
	}

	void SetDash (const char *pattern, int offset = 0)
	{
		cAString neu;
		if (!pattern)
			pattern = "";
		neu.Format ("[%s]%d", pattern, offset);
		if (gsLineDash != neu) {
			gsLineDash = neu;
			gsLineDashChanged = 1;
		}
	}

	cAString checkLineStyles ()
	{
		cAString s;
		if (gsStrokeColorChanged) {
			s.FormatAdd ("%s RG%s", gsStrokeColor, NL);
			gsStrokeColorChanged = 0;
		}
		if (gsLineWidthChanged) {
			s.FormatAdd ("%s w%s", roundmax (gsLineWidth, 2).Get (), NL);
			gsLineWidthChanged = 0;
		}
		if (gsLineCapChanged) {
			s.FormatAdd ("%d J%s", gsLineCap, NL);
			gsLineCapChanged = 0;
		}
		if (gsLineJoinChanged) {
			s.FormatAdd ("%d j%s", gsLineJoin, NL);
			gsLineJoinChanged = 0;
		}
		if (gsLineDashChanged) {
			s.FormatAdd ("%s d%s", gsLineDash.Get (), NL);
			gsLineDashChanged = 0;
		}
		return s;
	}

	cAString checkFillStyle ()
	{
		cAString s;
		if (gsFillColorChanged) {
			s.Format ("%s rg%s", gsFillColor.Get (), NL);
			gsFillColorChanged = 0;
		}
		return s;
	}

	cAString evalPath ()
	{
		cAStringArray ar;
		split (ar, path, " ");
		path.Reset ();	// Pfad leeren
		cAString s;
		int i, max = ar.GetCount ();
		for (i = 0; i < max - 1; ) {
			switch (ar.GetIt (i)->Get ()[0]) {
			case 'm':
				s.FormatAdd ("%s %s m%s"
						, ar.GetIt (i + 1)->Get (), ar.GetIt (i + 2)->Get (), NL);
				i += 3;
				break;
			case 'l':
				s.FormatAdd ("%s %s l%s"
						, ar.GetIt (i + 1)->Get (), ar.GetIt (i + 2)->Get (), NL);
				i += 3;
				break;
			case 'c':
				s.FormatAdd ("%s %s %s %s %s %s c%s"
						, ar.GetIt (i + 1)->Get (), ar.GetIt (i + 2)->Get ()
						, ar.GetIt (i + 3)->Get (), ar.GetIt (i + 4)->Get ()
						, ar.GetIt (i + 5)->Get (), ar.GetIt (i + 6)->Get (), NL);
				i += 7;
				break;
			case 'h':
				s += 'h' + NL;
				i += 1;
				break;
			}
		}
		return s;
	}

	cAString Stroke ()
	{
		cAString s;
		s = optAdd (s.Format ("%s%sS", checkLineStyles ().Get (), evalPath ().Get ()).Get ());
		return s;
	}

	cAString ClosePathAndStroke ()
	{
		cAString s;
		s = optAdd (s.Format ("%s%ss", checkLineStyles ().Get (), evalPath ().Get ()).Get ());
		return s;
	}

	cAString Fill ()
	{
		cAString s;
		s = optAdd (s.Format ("%s%sf", checkFillStyle ().Get (), evalPath ().Get ()).Get ());
		return s;
	}

	cAString FillAndStroke ()
	{
		cAString s;
		s = optAdd (s.Format ("%s%s%sB", checkLineStyles ().Get (), checkFillStyle ().Get (), evalPath ().Get ()).Get ());
		return s;
	}

	cAString ClosePathFillAndStroke ()
	{
		cAString s;
		s = optAdd (s.Format ("%s%s%sb", checkLineStyles ().Get (), checkFillStyle ().Get (), evalPath ().Get ()).Get ());
		return s ;
	}

	cAString PaintBitmap (double x, double y, double dx, double dy
						, int sourceW, int sourceH
						, const char *colorSpace, int bitsPerComponent
						, const char *filter, const char *data, const char *extra)
	{
		cAString s;
		s = optAdd (
			s.Format ("q%s"		// Save GS + NL
						"%s 0 0 %s %s %s cm%s"	// dx, dy, x, y, NL
						"BI%s"		// + NL
						"/W %d%s"	// + sourceW + NL
						"/H %d%s"	// + sourceH + NL
						"/CS %s%s"	// + colorSpace + NL	// /G /RGB /CMYK /I
						"/BPC %d%s"	// + bitsPerComponent + NL	// /G (1,2,4,8) /RGB (8) /CMYK (8), neuerdings auch 16
						"/F %s%s"	// + filter + NL	// /AHx
						"%s%s"		//+ (extra.length ? (extra + NL) : "")
						"ID %s%s"	// + data + NL	//		7F FF DD FF FF 7F DD FF>
						"EI%s"		// + NL
						"Q%s"
						, NL
						, roundmax (dx, 2).Get (), roundmax (dy, 2).Get (), roundmax (x, 2).Get (), roundmax (y, 2).Get (), NL
						, NL
						, sourceW, NL
						, sourceH, NL
						, colorSpace, NL	// /G /RGB /CMYK /I
						, bitsPerComponent, NL	// /G (1,2,4,8) /RGB (8) /CMYK (8), neuerdings auch 16
						, filter, NL	// /AHx
						, extra ? extra : "", NL
						, data, NL	//		7F FF DD FF FF 7F DD FF>
						, NL
						, NL).Get ());
		return s;
	}

	//--- AddBookmark ("Title", "Level:0; open:1; /Dest [{Page0} /XYZ 1 800 1.4]")
	//--- AddBookmark ("Title", "Level:0;         /A << /Type /Action /S /GoTo /D [{Page0} /XYZ 1 800 1.4] >>")
	void AddBookmark (const char *title, const char *options)
	{
		if (Pdf && title && *title) {
			cBookmark *bkm = new cBookmark (title, 0, 0, "");
			//--- die Optionen auswerten
			cAString Options = options;
			{
				options = Options;
				char *s1, *s2, *s3, *s4;	//	options = options.replace (/;\s+/g, ';');
				while ((s1 = strstr (options, "; " )) || (s2 = strstr (options, ";\t"))
					||  (s3 = strstr (options, ";\r")) || (s4 = strstr (options, ";\n"))) {
					if (s1)
						strcpy (s1 + 1, s1 + 2);
					else if (s2)
						strcpy (s2 + 1, s2 + 2);
					else if (s3)
						strcpy (s3 + 1, s3 + 2);
					else if (s4)
						strcpy (s4 + 1, s4 + 2);
				}
			}
			cAStringArray opts;
			split (opts, options, ";");
			int i, max = opts.GetCount ();
			for (i = 0; i < max; i++) {
				const char *s = opts.GetIt (i)->Get ();
				if (0 == strncmp (s, "Level:", 6))
					bkm->Level = atoi (s + 6);
				else if (0 == strncmp (s, "open:", 5))
					bkm->open = atoi (s + 5);
				else if (s [0] == '/')
					bkm->Dest.Set (s);	// -> /Dest [{Page0} /XYZ 1 800 1.4]
			}
			if (bkm->Level < 0)	// Level korrigieren
				bkm->Level = 0;
			if (Pdf->bookmarks->getLength ()) {	// Hierarchie prfen
				cBookmark *prevBkm = (cBookmark *)Pdf->bookmarks->get (Pdf->bookmarks->getLength () - 1);
				if (bkm->Level > prevBkm->Level + 1)
					bkm->Level = prevBkm->Level + 1;
			}
			Pdf->bookmarks->push (bkm);
		}
	}
/*	22 0 obj	<<                93 0 obj <<             99 0 obj <<					99 o obj
		/Type /Annot					/Type /Annot				/Type /Annot				/Type /Annot
		/Subtype /Text					/Subtype /Link				/Subtype /Movie			/Subtype /Sound
		/Rect [266 116 430 204]		/Rect [71 717 190 734]	/Rect [71 717 190 734]
		/Contents (text for two)	/Border [16 16 1]      	/Movie <<					/Sound <<
	>>	endobj							/Dest [3 0 R /Fit]			/F (file.mpg)				/F (file.wav)
                              >> endobj						>>								>>
																		>> endobj					>> endobj
	*/
	void AddAnnot (const char *subtype, const char *spec)
	{
		if (!Pdf || !Page)
			return;
		if (strcmp (subtype, "/Text") && strcmp (subtype, "/Link")
		 && strcmp (subtype, "/Movie") && strcmp (subtype, "/Sound"))
			return;
		if (!spec || (*spec != '/'))
			return;

		cAString temp;
		cArray *annots = (cArray *)Page->getValueOfKey ("/Annots");
		if (annots)			// ist jetzt noch ein cPdfString-Object
			annots = (cArray *)Pdf->asObj ((cPdfString *)annots);
		else {
			annots = Pdf->newPdfArrayObj ();
			Page->addKey ("/Annots", Pdf->asRObj (annots).Get ());
		}
		cPdfDict *annot = Pdf->newPdfDictObj ("/Annot");
		annot->addKey ("/Subtype", subtype);

		//--- Den restlichen dict-Inhalt holen.
		cAString key, value;
		int i, max = strlen (spec);
		for (i = 1; (i < max) && (spec [i] > ' '); i++);
		key.Set (spec, 0, i);
		for ( ; (i < max) && (spec [i] <= ' '); i++);
		value.Set (spec, i, max);
		annot->addKey (key, value);

		//--- in /Annots eintragen
		annots->add (new cPdfString (Pdf->asRObj (annot).Get ()));
	}
};

#endif //PdfDoc_hpp