
//--- PdfTest.cpp   Konsolenanwendung
//--- Copyright Thilo Brai, 2012
//--- Creative Commons Lizenz: CC-by-sa
//
//--- PdfTest.cpp testet die Funktionalitt der Klassen PdfDoc und PdfGraphic.
//--- erzeugt im aktuellen Verzeichnis die Dateien:
//--- Hallo_Welt.pdf und
//--- Mit_allem_Schnickschnack.pdf

#include <stdio.h>	// printf, vprintf...
#include <fcntl.h>	//	_O_WRONLY
#include <sys/stat.h>// _S_IREAD | _S_IWRITE
#include <io.h>		// _open ()
#include <string.h>
#include <time.h>
#include <stdlib.h>	// strtod ()
#include <ctype.h>	// isdigit ()

#include "PdfGraphic.hpp"	// Das reicht fr die reine Pdf-Erstellung (Beispiel 1)
#include "cWordLayout.hpp"	// Text in Zeilen aufteilen (Beispiel 2)


//--- Fehlermeldungen ausgeben
void alert (const char *text)
{
	if (text)
		printf ("%s\r\n", text);
}

//--- Datei schreiben
//--- Dateiname nummerieren, falls die Datei nicht geffnet werden kann.
void write_text_to_file (const char *filename, const char *text)
{
	cAString numFile = filename;
TrayAgain:
	int file = _open (filename, _O_WRONLY | _O_BINARY | _O_CREAT | _O_TRUNC, _S_IREAD | _S_IWRITE);
	if (file == -1) {
		if (strrchr (numFile, '.')) {
			*strrchr (numFile, '.') = 0;

			char *p = strchr (numFile, 0);
			int num = 0;
			if (isdigit (*(p - 1))) {
				while ((numFile.Get () < p) && isdigit (*(p - 1)))
					p--;
				num = atoi (p);
				*p = 0;
			}
			if (num < 100) {
				numFile.FormatAdd ("%u.pdf", num + 1);
				filename = numFile.Get ();
				goto TrayAgain;
			}
		}
	}
	if (file != -1) {
		int len = strlen (text);
		_write (file, text, len);
		_close (file);
	} else {
		cAString temp;
		alert (temp.Format ("Datei '%s' nicht geffnet!", filename).Get ());
	}
}

//--- Beispiel 1
//--- Hallo Welt als Pdf
//--- Eine Version mit Zeiger
void MakePDF_HalloWelt_Ptr (const char *filename)
{
	//--- Die Pdf-Verwaltung erzeugen
	cPdfDoc Pdf;
	Pdf.init ();

	//--- Die Grafikeinheit an die Pdf-Verwaltung koppeln
	cPdfGraphic *g = new cPdfGraphic (Pdf);
	g->InitPage (0);             // die erste Seite ffnen

	g->SetFont ("/TimesNewRoman");
	g->SetFillColor ("0 0 .6");		  // blauer Text
	g->TextOutBlock (100, 700, "Hallo Welt.");	// links oben auf dem A4-Blatt

	//--- g dient nur dazu, einige Pdf-Regularien einzuhalten.
	//--- Die Objekte, Streams, Arrays, Dictionaries residieren in PdfDoc.
	//--- Kurzzeitig werden Infos in PdfGraphic gepuffert.
	//--- TextOutBlock (), Stroke (), Fill () etc. flusht die Infos.
	//--- Wenn wir jetzt lschen, geht nichts von den Pdf-Infos kaputt.
	delete g;

	//--- Zum Erzeugen des Pdf-Textes, ruft man dann noch createPdf ()
	cAString pdfString;
	Pdf.createPdf (pdfString);

	write_text_to_file (filename, pdfString.Get ());
}

//--- Hallo Welt als Pdf
void MakePDF_HalloWelt (const char *filename)
{
	//--- Die Pdf-Verwaltung erzeugen
	cPdfDoc Pdf;
	Pdf.init ();

	//--- Die Grafikeinheit g an die Pdf-Verwaltung koppeln
	cPdfGraphic g (Pdf);
	g.InitPage (0);             // die erste Seite ffnen

	g.SetFont ("/TimesNewRoman");
	g.SetFillColor ("0 0 .6");  // blauer Text
	g.TextOutBlock (100, 700, "Hallo Welt.");

	//--- Zum Erzeugen des Pdf-Textes, ruft man dann noch createPdf ()
	cAString pdfString;
	Pdf.createPdf (pdfString);

	write_text_to_file (filename, pdfString.Get ());
}

//--- Beispiel 2
//--- Groe Demo, zwei Seiten.
//--- Demonstriert die Text-, Grafik-, Bitmap-, Bookmarks- und Annotation-Features.
void MakePDF (const char *filename)
{
	cPdfDoc Pdf;
	Pdf.init ();

	cPdfGraphic g (Pdf);
	g.InitPage (0);

	double linkerRand = mm2pt (25);
	double Zeile1 = mm2pt (280);
	double Zeilenhoehe = 15;
	double Breite =  mm2pt (160);	// 210 - 2 * 25

	//--- Eine dnne linie ziehen
	g.SetStrokeColor ("1 .7 .7"); // helles Rot
	g.SetLineCap (1); // runden Enden
	g.SetLineJoin (1); // runde Ecken
	g.SetLineWidth (.25);
	g.Line (linkerRand, Zeile1, linkerRand + Breite, Zeile1);
	g.Stroke ();	// Die Linie wird erst bei Stroke () gezeichnet.


	//--- Noch einige Linien malen
	for (int i = 0; i < 10; i++) {
		g.Line (linkerRand + i * 30, Zeile1 - i * Zeilenhoehe
			, linkerRand + Breite, Zeile1 - i * Zeilenhoehe);
		g.Line (linkerRand + Breite - i * Zeilenhoehe, Zeile1
			, linkerRand + Breite - i * Zeilenhoehe, Zeile1 - Zeilenhoehe * (20 - i));
	}
	g.Stroke ();


	//--- Eine Ellipse
	g.SetStrokeColor (".7 .7 1"); // helles Blau
	g.Ellipse (linkerRand + Breite - 60, Zeile1 - 45, 60, 45);
	g.Stroke ();


	//--- Eine Schrift auswhlen
	g.SetFont ("/TimesNewRoman"); //	g.SetFont ("/Times-Roman", "/Type1");
	g.SetFontSize (16);
	g.SetFillColor (".2 .2 .7");	// Blau
	g.TextOutBlock (linkerRand, Zeile1 - Zeilenhoehe, "Ein Text in 16 pt Times (mit Klammern).");


	//--- Einige Bookmarks einfgen. Einige aufklappen, andere nicht.
	//--- Das erste Argument ist der Titel, das Zweite das Einrck-Level, dann offen/geschlossen.
	//--- Hinten steht der Aktion-Key mit den Anweisungen (ist in der Pdf-Referenz erklrt).
	//--- Nach dem erstem Aktion-Key muss (!) ein Leerzeichen stehen. Mit {PageX} kann man die 
	//--- Pdf-Seiten-Objekts ansprechen, die in der Pdf dann als "14 0 R" u. . stehen.
	g.AddBookmark ( "Seite 1 alles (L0.1)",         "Level:0; /Dest [{Page0} /Fit]"          );
	g.AddBookmark ( "Seite 1 (L0.2)"      , "open:1; Level:0; /Dest [{Page0} /FitH 842]"     );
	g.AddBookmark  ("Oben (L1.1)"         ,         "Level:1; /Dest [{Page0} /XYZ -1 843 2.5]");
	g.AddBookmark  ("Mitte (L1.2)"        ,         "Level:1; /Dest [{Page0} /XYZ -1 500 3]"  );
	g.AddBookmark  ("Unten (L1.3)"        ,         "Level:1; /Dest [{Page0} /XYZ -1 150 3.5]");
	g.AddBookmark ( "Seite 2 alles (L0.3)",         "Level:0; /Dest [{Page1} /Fit]"          );
	g.AddBookmark  ("Oben (L1.4)"         ,         "Level:1; /Dest [{Page1} /XYZ -1 844 2.5]");
	g.AddBookmark  ("Mitte (L1.5)"        ,         "Level:1; /Dest [{Page1} /XYZ -1 500 3]"  );
	g.AddBookmark  ("Unten (L1.6)"        , "open:1; Level:1; /Dest [{Page1} /XYZ -1 150 3.5]");
	g.AddBookmark  ("Rechts unten (L2.1)" ,         "Level:2; /Dest [{Page1} /XYZ 500 150 4]");
	//--- Gleichbedeutend knnte man als Action auch schreiben:
	//    /A << /Type /Action /S /GoTo /D [{Page0} /XYZ 1 800 1.4] >>

	g.SetFont ("/Arial");
	g.SetFontSize (12);
	g.SetFillColor ("0 0 .4"); // Dunkelblau
	g.TextOutBlock (linkerRand, Zeile1 - 3 * Zeilenhoehe, "Ein Text in 12 pt Arial.");

	g.SetFontSize (10);
	g.SetFillColor ("0 0 0");
	g.TextOutBlock (linkerRand, Zeile1 - 5 * Zeilenhoehe, "Eine weitere Zeile mit einigen Umlauten  und dem -Zeichen.");

	g.SetLineWidth (1);
	g.SetStrokeColor (1, .5, .5); // Rot
	g.SetFillColor (.5, .5, 1); // Blau
	g.Rect (40, 803, 47, 810);
	g.FillAndStroke ();

	g.SetFillColor (.5, .7, .5); // Grn
	g.Rect (43, 783, 47, 793);
	g.Fill ();
	
	g.SetStrokeColor (.9, .5, .9); // dunkles Lila
	g.Rect (40, 762, 48, 766);
	g.Stroke ();

	//--- Einen Textblock mit verschiedenen Schriftgren erstellen
	{
		g.writeAtOnce = 0;
		cPdfDict *fontDict = g.SetFont ("/TimesNewRoman");
		cPdfString *fontTimes = (cPdfString *)fontDict->getValueOfKey ("/Name");	// z. B. /Time0 (! Zahl am Ende)
		fontDict = g.SetFont ("/Arial");
		cPdfString *fontArial = (cPdfString *)fontDict->getValueOfKey ("/Name");	// z. B. /Aria1 (! Zahl am Ende)
		g.SetFontSize (40);
		//--- Textblcke beginnen mit dem BeginText-Marker
		cAString s, temp;
		s.Format ("BT%s", NL);				// + NL;
		//--- 40 pt Arial, schwarz, x = 50, y = 700
		s.FormatAdd ("0 0 0 rg%s"			// + NL;
						"%s%s"					// x, y, (A) + NL;
						, NL
						, g.TextOut (linkerRand, Zeile1 - 9 * Zeilenhoehe, "A"), NL);
		//--- 32 pt Arial, rot
		s.FormatAdd ("%s 32 Tf%s"			// fontArial + NL
						".98 .55 .55 rg%s"	// + NL
						"(B) Tj%s"				// + NL		// Um in der Zeile zu bleiben, Text mit Tj ausgeben.
						, fontArial->string.Get (), NL
						, NL, NL);
		//--- 50 pt Arial, grn
		s.FormatAdd ("%s 50 Tf%s"			//	+ fontArial + " 50 Tf" + NL
						".45 .72 .45 rg%s"	// + NL
						"(C) Tj%s"				// + NL
						, fontArial->string.Get (), NL
						, NL, NL);
		//--- 32 pt Arial, blau
		s.FormatAdd ("%s 32 Tf%s"			// fontArial + NL
						".55 .55 .75 rg%s"	// + NL
						"(D) Tj%s"				// + NL
						, fontArial->string.Get (), NL
						, NL, NL);
		//--- 10 pt Arial, grn, hochgestellt
		s.FormatAdd ("%s 10 Tf%s"			//	fontArial + NL
						"20 Ts%s"				//	+ NL	(hher)
						"0 .5 0 rg%s"			// + NL
						"(sup) Tj%s"			// + NL
						, fontArial->string.Get (), NL
						, NL, NL, NL);
		//--- 32 pt Arial, blau, 
		s.FormatAdd ("%s 32 Tf%s"			//	fontArial +  + NL
						".55 .55 .75 rg%s"	// + NL
						"0 Ts%s"					// (wieder normal) + NL
						"( normal) Tj%s"		// + NL
						,fontArial->string.Get (), NL
						, NL, NL, NL);
		//--- 14 pt Arial, rot, tiefergestellt
		s.FormatAdd ("%s 14 Tf%s"			//	fontArial +  + NL
						"-10 Ts%s"				// + NL (tiefer)
						"1 0 0 rg%s"			// + NL	// Rot
						"(sub) Tj%s"			// + NL
						,fontArial->string.Get (), NL
						, NL, NL, NL);
		//--- 50 pt Times, grn
		s.FormatAdd ("%s 50 Tf%s"			//fontTimes +  + NL
						".45 .72 .45 rg%s"	// + NL
						"0 Ts%s"					// (wieder normal) + NL
						"( EfGhIjkl) Tj%s"	// + NL
						,fontArial->string.Get (), NL
						, NL, NL, NL);
		//--- und den EndText-Marker nicht vergessen
		s += "ET";
		g.Add (s);
		g.writeAtOnce = 1;
	}

	//--- noch ein bisschen Mondrian
	{
		tCollection <const char *> col;
		col.Add (".2 .2 .2");		// Schwarz
		col.Add (".97 .88 .08");	// Gelb
		col.Add (".96 .3 .07");		// Rot
		col.Add (".43 .58 .91");	// Blau
		
		const char **cols = col.GetItAsPtr (0);

		g.SetLineWidth (3);

		srand ((unsigned)time (NULL));

		for (int i = 0; i < 20; i++) {
			double x = linkerRand + (rand () / double (RAND_MAX)) * (Breite - 150);
			double y = linkerRand + (rand () / double (RAND_MAX)) * Breite;
			double dx = 10 + (rand () / double (RAND_MAX)) * 150;
			double dy = 10 + (rand () / double (RAND_MAX)) * 150;
			g.SetFillColor (cols [rand () & 0x03]);
			g.SetStrokeColor (cols [rand () & 0x03]);
			g.Rect (x, y, x + dx, y + dy);
			g.FillAndStroke ();
		}
	}


	//--- Eine zweite Seite erzeugen
	g.InitPage (1);
	g.SetFillColor (".9 .9 .8");	// Senfgelb
	g.SetStrokeColor (".6 .6 .4");
	g.SetLineWidth (1);	// 1 pt
	int y = 820;
	g.Rect (30, y, 560, 20);	//595 842
	g.FillAndStroke ();
	g.SetFont ("/Arial,Bold");
	g.SetLineWidth (.5);
	g.SetTextRenderingMode (2);
	g.SetFontSize (30);
	g.SetFillColor ("1 1 1");	// Wei
	g.TextOutBlock (linkerRand - 1, y -= 40, "Eine zweite Seite.");

	g.SetTextRenderingMode (0);
	g.SetFontSize (20);
	g.SetFillColor (".6 .6 .4");
	g.TextOutBlock (linkerRand, y -= 40, "Mit einer Bitmap.");

	g.TextOutBlock (linkerRand, 410, "Mitte.");
	g.TextOutBlock (linkerRand, 50, "Unten.");
	g.TextOutBlock (linkerRand + Breite - 135, 50, "Rechts unten.");


	//--- Eine Pixelgrafik ausgeben.
	//--- Die Abmessung ist W = 10, H = 7.
	//--- Die Bits einer Bilderzeile mssen ein Vielfaches von 8 sein.
	//--- Zu pN1: Ein Punkt ergibt Wei (== 1), Leerstellen werden gelscht.
	g.PaintBitmap (100, 700, 40, 28, 10, 7, "/G", 1, "/AHx", 
		    // 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6
		pN1 (	". . . . . . . . . . - - - - - -"
				". X X . . . . . . . - - - - - -"
				"X . . . X . . . X . - - - - - -"
				"X . . X X X . X X X - - - - - -"
				"X . . . X . . . X . - - - - - -"
				". X X . . . . . . . - - - - - -"
				". . . . . . . . . . - - - - - -"
			).Get ()	// + '>' Stream-Ende-Marker wird in der C++-Version in pN1 () addiert
			, "/Interpolate true");


	//--- Einen Textblock formatieren. Ansi 173 ergibt eine weiche Trennung
	const char *blindtext = 
					"Achtung! Dieser Blindtext wird gerade durch 130 Millionen Rezeptoren "
					"Ihrer Netzhaut erfasst. Die Zellen werden dadurch in einen "
					"Erregungszustand versetzt, der sich ber den Sehnerv in dem "
					"hinteren Teil Ihres Gehirns ausbreitet. Von dort aus bertrgt "
					"sich die Erregung in Sekundenbruchteilen auch in andere Bereiche "
					"Ihres Grohirns. Ihr Stirnlappen wird stimuliert. Von dort aus "
					"gehen jetzt Willensimpulse aus, die Ihr zentrales Nervensystem in "
					"konkrete Handlungen umsetzt. Kopf und Augen reagieren bereits. "
					;// der folge Texte sollte nicht mehr erscheinen, wenn layout () mit done aufgerufen wird
					//"     Sie folgen dem Text, nehmen die darin enthaltenen Informationen auf wie ein Schwamm. Nicht auszudenken, was mit Ihnen htte passieren knnen, wenn dieser Blindtext durch einen echten Text ersetzt worden wre.";
	g.SetFillColor (".96 .96 .9");	// zuerste einen Rahmen ziehen
	g.SetStrokeColor (".7 .7 .6");
	g.RectWH (linkerRand, 380, 220, 135);
	g.FillAndStroke ();
	//--- Zuerst als linksbndiger Absatz
	g.SetFont ("/Arial,Bold");
	g.SetFontSize (11);
	g.SetFillColor ("0 0 0");
	g.TextOutBlock (linkerRand, 385, "Linksbndiger Text:");

	cPdfDict *fontDict = g.SetFont ("/Arial");
	cPdfString *fontArial = (cPdfString *)fontDict->getValueOfKey ("/Name");	// z. B. /Aria2
	{
		cAString s;
		//--- Textblcke beginnen mit dem BeginText-Marker
		s.Format ("BT%s", NL);
		//--- 10 pt Arial, schwarz
		s.FormatAdd ("%s 10 Tf%s"
						"0 0 0 rg%s"
						, fontArial->string.Get (), NL
						, NL);
		//--- den Absatz formatieren
		int done;	// Anzahl der Zeichen, die ausgegeben wurden
		double rectAr [] = {linkerRand, 380, 220, 135};
		s += cWordLayout::layoutAbsatz (blindtext
							, rectAr	// {linkerRand, 380, 220, 135};
							, 10	//fontSize
							, cWordLayout::ArialWidths	//widths
							, 12	//lineHeight
							, 0	//lcbr: 0 = linksbndig, 1 = zentriert, 2 = Blocksatz, 3 = rechtsbndig
							, &done);	// done sollte 521 liefern
		s += "ET";
		g.Add (s);
	}

	//--- Jetzt als rechtsbndiger Absatz
	g.SetFont ("/Arial,Bold");
	g.SetFontSize (11);
	g.SetFillColor ("0 0 0");
	g.TextOutBlock (linkerRand + 230, 385, "Rechtsbndiger Text:");

	g.SetFont ("/Arial");	// Nur damit PdfGrafic merkt, dass wir im BT-Block umgeschaltet haben
	g.SetFillColor (".96 .96 .9");
	g.RectWH (linkerRand + 230, 380, 220, 135);
	g.FillAndStroke ();
	{
		cAString s;
		//--- Alles gleich, bis auf den letzten Parameter fr layoutAbsatz ()
		s.Format ("BT%s", NL);
		//--- 10 pt Arial, schwarz
		s.FormatAdd ("%s 10 Tf%s"
						"0 0 0 rg%s"
						, fontArial->string.Get (), NL
						, NL);
		//--- den Absatz formatieren
		int done;
		double rectAr [] = {linkerRand + 230, 380, 220, 135};
		s += cWordLayout::layoutAbsatz (blindtext
						, rectAr	//	(linkerRand + 230, 380, 220, 135)
						, 10	//fontSize
						, cWordLayout::ArialWidths	//widths
						, 12	//lineHeight
						, 3	//lcbr: 0 = linksbndig, 1 = zentriert, 2 = Blocksatz, 3 = rechtsbndig
						, &done);
		s += "ET";
		g.Add (s);
	}

	//--- Noch als zentrierter Absatz
	g.SetFont ("/Arial,Bold");
	g.SetFontSize (11);
	g.SetFillColor ("0 0 0");
	g.TextOutBlock (linkerRand, 225, "Zentrierter Text:");

	g.SetFont ("/Arial");	// Nur damit PdfGrafic merkt, dass wir im BT-Block umgeschaltet haben
	g.SetFillColor ("0 0 0");	// Die Fllfarbe im letzten Textblck ging an g vorbei!
	g.SetFillColor (".96 .96 .9");
	g.RectWH (linkerRand, 220, 220, 135);
	g.FillAndStroke ();
	{
		cAString s;
		//--- Alles gleich, bis auf den letzten Parameter fr layoutAbsatz ()
		s.Format ("BT%s", NL);
		//--- 10 pt Arial, schwarz
		s.FormatAdd ("%s 10 Tf%s"
						"0 0 0 rg%s"
						, fontArial->string.Get (), NL
						, NL);
		//--- den Absatz formatieren
		int done;
		double rectAr [] = {linkerRand, 220, 220, 135};
		s += cWordLayout::layoutAbsatz (blindtext
						, rectAr	// (linkerRand, 220, 220, 135)
						, 10	//fontSize
						, cWordLayout::ArialWidths	//widths
						, 12	//lineHeight
						, 1	//lcbr: 0 = linksbndig, 1 = zentriert, 2 = Blocksatz, 3 = rechtsbndig
						, &done);
		s += "ET";
		g.Add (s);
	}

	//--- Und ein Blocksatz
	g.SetFont ("/Arial,Bold");
	g.SetFontSize (11);
	g.SetFillColor ("0 0 0");
	g.TextOutBlock (linkerRand + 230, 225, "Blocksatz:");

	g.SetFont ("/Arial");	// Nur damit PdfGrafic merkt, dass wir im BT-Block umgeschaltet haben
	g.SetFillColor ("0 0 0");	// Die Fllfarbe im letzten Textblck ging an g vorbei!
	g.SetFillColor (".96 .96 .9");
	g.RectWH (linkerRand + 230, 220, 220, 135);
	g.FillAndStroke ();
	{
		cAString s;
		//--- Alles gleich, bis auf den letzten Parameter fr layoutAbsatz ()
		s.Format ("BT%s", NL);
		//--- 10 pt Arial, schwarz
		s.FormatAdd ("%s 10 Tf%s"
						"0 0 0 rg%s"
						, fontArial->string.Get (), NL
						, NL);
		//--- den Absatz formatieren
		int done;
		double rectAr [] = {linkerRand + 230, 220, 220, 135};
		s += cWordLayout::layoutAbsatz (blindtext
						, rectAr	// (linkerRand + 230, 220, 220, 135)
						, 10	//fontSize
						, cWordLayout::ArialWidths	//widths
						, 12	//lineHeight
						, 2	//lcbr: 0 = linksbndig, 1 = zentriert, 2 = Blocksatz, 3 = rechtsbndig
						, &done);
		s += "ET";
		g.Add (s);
	}


	//--- Zum guten Schluss noch ein einfacher Kommentar und ein Link auf die erste Seite.
	g.SetFont ("/Arial,Bold");
	g.SetFontSize (11);
	g.SetFillColor ("0 0 0");
	g.TextOutBlock (200, 655, "Ein Kommentar:");
	{
		cAString s;
		g.AddAnnot ("/Text"
			, s.Format ("/Rect [200 600 400 650]%s"
							"/Contents (Das ist ein Kommentar)", NL).Get ());
	}

	//--- Und eine Verknpfung
	g.SetFont ("/Arial,BoldItalic");
	g.SetFontSize (12);
	g.SetFillColor (".5 0 0");	// Dunkelrot
	g.TextOutBlock (205, 505, "Eine Verknpfung zur ersten Seite");
	{
		cAString s;
		g.AddAnnot ("/Link"
			, s.Format ("/Rect [200 500 405 518]%s"
							"/Border [16 16 1]%s"
							"/Dest [{Page0} /Fit]", NL, NL).Get ());
	}

	//--- wenn man mag, kann man noch wichtige Infos in die PDF eintragen
	Pdf.Info->addKey ("/Title",   Win2PdfEncoding ("(Demo fr den C++ PDF-Kreator)"));
	//Pdf.Info.addKey ("/Subject",  Win2PdfEncoding ("(Das Thema)"));
	//Pdf.Info.addKey ("/Author",   Win2PdfEncoding ("(Der Verfasser)"));
	//Pdf.Info.addKey ("/Keywords", Win2PdfEncoding ("(Die Stichwrter)"));
	//Pdf.Info.addKey ("/Creator",  Win2PdfEncoding ("(Erstellt mit JavaScript)"));

	//--- jetzt die PDF erzeugen und ins HTML-Formular schreiben
	cAString pdfString;
	Pdf.createPdf (pdfString);

	write_text_to_file (filename, pdfString.Get ());
}


int main (int argc, char* argv[])
{
	printf("Der Pdf C++ Kreator\r\n");

	printf("Hallo Welt...");
	MakePDF_HalloWelt ("Hallo_Welt.pdf");	// Dateiname: Hallo_Welt.pdf
	printf("\r\n");

	printf("Pdf mit Text, Grafik\n");
	MakePDF ("Mit_allem_Schnickschnack.pdf");	// Dateiname: Mit_allem_Schnickschnack.pdf
	printf("\r\n");

	return 0;
}


