
//--- cString.hpp
//--- Copyright Thilo Brai, 2012
//--- Creative Commons Lizenz: CC-by-sa
//
//--- Enthlt einige Klassen zur Speicherautomation 
//--- cAString (AnsiString), tPtrArray, cAStringArray, split (), tCollection
//--- Nichts fr Unicode, weil Pdf mit Unicode auf Kriegsfu steht.

#ifndef cString_hpp
#define cString_hpp

#include <stdio.h>	// _vsnprintf ()
#include <stdarg.h>	// _vsnprintf ()

//--- class AnsiString
class cAString {
public:
	cAString () { s = 0; }
	cAString (const char *str, int max = -1) { s = 0; Set (str, max); }
	cAString (cAString &string) { s = 0; Set (string); }
	cAString (cAString *string) { s = 0; if (string) Set (string->Get ()); }
	~cAString () { Reset (); }
private:
	char *s;
public:
	void Reset () {
		if (s) {
			delete [] s;
			s = 0;
		}
	}

	const char *Get () const {
		if (s)
			return s;
		return "";
	}

	unsigned CharAt (int index) const {
		int len = SaveStrlen (s);
		if (index < len)
			return ((unsigned char *)s) [index];
		return 0;
	}

	unsigned LastChar () const {
		if (s)
			return CharAt (SaveStrlen (s) - 1);
		return 0;
	}

	const cAString &Set (const char *str, int max = -1) {
		max = SaveStrlen (str, max);
		if (max) {
			if (str == s) {	// Selbstzuweisung
				s [max] = 0;
			} else {
				Reset ();
				s = new char [max + 1];
				strncpy (s, str, max);
				s [max] = 0;
			}
		} else if (s)
			Reset ();
		return *this;
	}
	const cAString &Set (const char *str, int start, int count) {
		if (str) {
			int len = cAString::SaveStrlen (str, start + count);
			if (len && (start < len))
				return Set (str + start, count);
			else
				Reset ();
		}
		return *this;
	}
	const cAString &Set (cAString &string, int max = -1) { return Set (string.Get (), max); }
	const cAString &Set (cAString &string, int start, int count) { return Set (string.Get (), start, count); }

	const cAString &SetAt (int index, char ch) {
		unsigned len = SaveStrlen (s);
		if ((unsigned)index >= len)
			return Add (&ch, 1);
		else if (s)
			s [index] = ch;
		else {
			if (ch)
				return Set (&ch, 1);
		}
		return *this;
	}

	const cAString &Add (cAString &string, int max = -1) { return Add (string.Get (), max); }
	const cAString &Add (const char *str, int max = -1) {
		if (str) {
			max = SaveStrlen (str, max);
			if (max) {
				int len1 = SaveStrlen (s);
				if (len1) {
					char *n = new char [len1 + max + 1];
					strcpy (n, s);
					strncpy (n + len1, str, max);
					n [len1 + max] = 0;
					delete s;
					s = n;
				} else
					Set (str, max);
			}
		}
		return *this;
	}
	const cAString &Add (const char *str, int start, int count) {
		if (str) {
			if (start < 0)
				start = 0;
			if (count > 0) {
				int len = SaveStrlen (str, start + count);
				if (start < len) {
					if (start + count > len)
						count = len - start;
					Add (str + start, count);
				}
			}
		}
		return *this;
	}
	const cAString &Add (char ch) { return Add (&ch, 1);	}

	//--- fr operator +
	//--- this ist leer!
	void ConcatCopy (const char *str1, const char *str2) {
		int len1 = SaveStrlen (str1);
		int len2 = SaveStrlen (str2);
		if (len1 && len2) {
			Reset ();
			s = new char [len1 + len2 + 1];
			strcpy (s, str1);
			strcpy (s + len1, str2);
		} else if (len1)
			Set (str1);
		else if (len2)
			Set (str2);
		else
			Reset ();
	}

	int IsEqual (const char *str) {
		unsigned len = SaveStrlen (str);
		if (!len)
			return (len == SaveStrlen (s, 1));
		if (s)
			return (0 == strcmp (s, str));
		return 0;
	}

	unsigned Length () { return SaveStrlen (s); }
	static unsigned SaveStrlen (const char *str, int max = -1) {
		if (str && *str) {
			if (max == -1)
				return strlen (str);
			int i;
			for (i = 0; (i < max) && str [i]; i++);
			return i;
		}
		return 0;
	}

	const cAString &Replace (char Old, char New) {
		if (s && *s && (Old != New)) {
			char *f = s;
	 		while (f = strchr (f, Old))
				*f = New;
		}
		return *this;
	}

	const cAString &Replace (const char *oldS, const char *newS) {
		unsigned int i, len, nlen, start, count;

		if (!oldS || !*oldS)
			return *this;
		if (!newS)
			newS = "";

		len  = SaveStrlen (oldS);
		nlen = SaveStrlen (newS);

		count = 0;
		start = 0;
		while ((i = Find (oldS, start)) != unsigned(-1)) {
			CutChars (i, len);
			if (nlen)
				InsertString (i, newS);
			start = i - len + nlen;
			count++;
		}
		return *this;
	}

	unsigned int Find (const char *find, unsigned int start) {
		const char *found;
		if (find && *find && (SaveStrlen (s) > start)) {
			if ((found = strstr (s + start, find)) != 0)
				return (unsigned)(found - s);
		}
		return (unsigned int)-1;
	}

	const cAString &InsertString (unsigned int index, const char *insertString, unsigned int insertMax = -1)
	{
		if (!insertString || !*insertString || !insertMax)	// 0-Pointer oder 0-Lnge
			return *this;

		unsigned int oldLen;

		if (insertMax = (unsigned)-1)
			insertMax = SaveStrlen (insertString);

		oldLen = SaveStrlen (s);

		if (oldLen <= index) {
			Add (insertString);
			return *this;
		}

		char *newString = new char [oldLen + insertMax + 1];
		if (newString) {
			if (index)
				strncpy (newString, s, index);
			strncpy (newString + index, insertString, insertMax);
			strcpy  (newString + index + insertMax, s + index);
			if (s)
				delete [] s;
			s = newString;
		}
		return *this;
	}

	const cAString &CutChars (unsigned int start, unsigned int len) {
		if (!len)
			return *this;

		unsigned int oldLen = SaveStrlen (s);
		if (oldLen <= start)
			return *this;

		if (start + len >= oldLen)
			s [start] = 0;
		else
			strcpy (s + start, s + start + len);

		return *this;
	}

	operator const char * ()                             { return Get (); }
	const cAString &operator =  (const char     *str   ) { return Set (str);	}
	const cAString &operator =  (      char      ch    ) { return Set (&ch, 1); }
	const cAString &operator =  (const cAString &string) { return Set (string.Get ()); }
	const cAString &operator += (      char      ch    ) { return Add (ch); }
	const cAString &operator += (const char     *str   ) { return Add (str); }
	const cAString &operator += (const cAString &string) { return Add (string.Get ()); }
	int             operator == (const char     *str   ) { return IsEqual (str); }
	int             operator == (const cAString &string) { return IsEqual (string.Get ()); }

	const cAString &Format (const char *format, ...) {
		int count, size = 1024;
		char *ns = new char [size];
		va_list list;

		do {
			va_start (list, format);
			#ifdef _MBCS
				count = _vsnprintf (ns, size - 1, format, list);
			#else
				count = _vsnwprintf (ns, size - 1, format, list);
			#endif
			va_end (list);

			ns [size - 1] = 0;	// zur Sicherheit terminieren

			if (count == -1)  {
				delete [] ns;
				size *= 2;
				ns = new char [size];
			}
		} while (count == -1);

		Set (ns);
		delete ns;
		return *this;
	}

	const cAString &FormatAdd (const char *format, ...) {
		int count, size = 1024;
		char *ns = new char [size];
		va_list list;

		do {
			va_start (list, format);
			#ifdef _MBCS
				count = _vsnprintf (ns, size - 1, format, list);
			#else
				count = _vsnwprintf (ns, size - 1, format, list);
			#endif
			va_end (list);

			ns [size - 1] = 0;	// zur Sicherheit terminieren

			if (count == -1)  {
				delete ns;
				size *= 2;
				ns = new char [size];
			}
		} while (count == -1);

		Add (ns);
		delete ns;
		return *this;
	}
};


inline cAString operator + (const cAString &string1, const cAString &string2)
{
	cAString s;
	s.ConcatCopy (string1.Get (), string2.Get ());
	return s;
}

inline cAString operator + (const cAString &string, const char *str)
{
	cAString s;
	s.ConcatCopy (string.Get (), str);
	return s;
}

inline cAString operator + (const char *str, const cAString &string)
{
	cAString s;
	s.ConcatCopy (str, string.Get ());
	return s;
}


template <class T> class tPtrArray {
public:
	tPtrArray  ()         { init (); }
	tPtrArray  (int size) { init (); alloc (size); }
	~tPtrArray ()         { RemoveAndDeleteAll (); }

private:
	unsigned int Count,
					Size;
	T *array;

public:
	int Add (T  pT){
		if (space ()) {
			array [Count++] = pT;
			return 1;
		}
		return 0;
	}
	//int AddNoDoubles (T  pT);
	//int Insert (T  pT, unsigned int atPos);
	//int Exchange (unsigned int pos1, unsigned int pos2);
	//T Exchange (unsigned int pos, T newT);
	T GetIt (unsigned int index) {
		if (index < Count)
			return array [index];
		return (T)0;	// Hart -- aber gltig.
	}
	T *GetArray () {	return array;	}
	unsigned int GetCount () { return Count; }
	int Alloc (unsigned int count) { return space (count); }
	//unsigned int IndexOf (T find);
	//unsigned int IndexOfFiLo (T find);
	//unsigned int IndexOfPart (void *partPtr, int startAt, int cmpLen);
	//unsigned int IndexOfPartFiLo (void *partPtr, int startAtByte, int cmpBytes);
	T Remove (unsigned int index){
		if (index < Count) {
			T ret = array [index];
			//--- Die Folgenden aufrcken...
			if (index + 1 < Count)
				memcpy (&array [index]
						, &array [index + 1]
						, sizeof (T) * (Count - index - 1));
			Count--;
			//--- prfen, ob er der erste ist.
			if (index == 0) {
				if ((Count == 0) && (Size > 8)) {
					if (array)
						delete array;
					init ();
				}
			}
			return ret;
		}
		return 0;
	}
	//void	RemoveAll ();
	void RemoveAndDelete (unsigned int index) {
		T t = Remove (index);
		if (t)
			delete t;
	}
	void	RemoveAndDeleteAll () {
		while (Count) {
			if (array [--Count])
				delete array [Count];
		}
	}

private:
	void init () {
		Size = Count = 0;
		array = 0;
	}
	int space () {
		if (Count + 1 >= Size)
			return alloc (Size + 8);
		return 1;
	}
	int space (unsigned int count){
		if (Count + count > Size)
			return alloc (Count + count);
		return 1;
	}
	int alloc (int newSize){
		T *nArray = new T [newSize];
		if (nArray) {
			if (array) {
				memcpy (nArray, array, sizeof (T) * Count);
				delete array;
			}
			array = nArray;
			Size = newSize;
			return 1;
		}
		return 0;
	}
	//int cmpPart (T *cmp, int startAt, int cmpLen, void *partPtr);
};


class cAStringArray : public tPtrArray <cAString *>
{
public:
	cAStringArray () { return; };
	~cAStringArray () { RemoveAndDeleteAll (); }

	int AddString (const char *string, int max = -1) {
		if (string && *string) {
			return Add (new cAString (string, max));
		} else
			return Add ((cAString *)0);
	}

	const char *GetString (unsigned int index) {
		cAString *pStr;
		const char *ret = "";

		if (index < GetCount ()) {
			pStr = GetIt (index);
			if (pStr != 0)
				ret = pStr->Get ();
		}
		return ret;
	}
};


inline int split (cAStringArray &ar, const char *s, const char *splitchars, int keepempty = 0)
{
	cAString cs;
	if (!s || !*s || !splitchars || !*splitchars)
		return 0;

	ar.RemoveAndDeleteAll ();

	const char *a, *e;
	for (a = e = s; *e; e++) {
		if (strchr (splitchars, *e)) {
			cs = "";
			while (a < e)
				cs += *a++;
			if (cs.Length () || keepempty)
				ar.AddString (cs);
			a = e + 1;
		}
	}
	if (a < e) {
		cs = a;
		if (cs.Length () || keepempty)
			ar.AddString (cs);
	}
	return ar.GetCount ();
}

template <class T> class tCollection {
	unsigned int Count,
		Size;
	T empty;
	T *array;
public:
	tCollection () { init (); }
	~tCollection () { RemoveAll (); }

	int	Add (T  pT) {
		if (space ()) {
			array [Count++] = pT;
			return 1;
		}
		return 0;
	}
	//int	AddNoDoubles (T  pT);
	//int	Insert (T  pT, unsigned int atPos);
	//int	Exchange (unsigned int pos1, unsigned int pos2);
	//T		Exchange (unsigned int pos, T newT);
	T		GetIt (unsigned int index) {
		if (index < Count)
			return array [index];
		return *(new T);
	}
	//int	GetIt (unsigned int index, T* t);
	T		*GetItAsPtr (unsigned int index) {
		if (index < Count)
			return &(array [index]);
		return 0;
	}
	unsigned int GetCount () { return Count; }
	//unsigned int GetSize () { return Size; }
	int	Alloc (unsigned int count) { return space (count); }
	//unsigned int IndexOf (T find);
	void	RemoveLast () {
		if (Count)
			Count--;
	}
	void RemoveAll () {
		if (array)
			delete array;
		init ();
	}

private:
	void init () {
		Size = Count = 0;
		array = 0;
	}
	int space () {
		if (Count + 1 >= Size) {
			if (Size < 64)
				return alloc (Size + 8);
			else
				return alloc (Size + Size / 2);
		}
		return 1;
	}
	int space (unsigned int count) {
		if (Count + count > Size)
			return alloc (Count + count);
		return 1;
	}
	int alloc (int newSize) {
	T *nArray = new T [newSize];
		if (nArray) {
			if (array) {
				memcpy (nArray, array, sizeof (T) * Count);
				delete array;
			}
			array = nArray;
			Size = newSize;
			return 1;
		}
		return 0;
	}
};

#endif
