
//--- WordLayout.hpp
//--- Copyright Thilo Brai, 2012
//--- Creative Commons Lizenz: CC-by-sa
//
//--- Klasse mit Wordbreak-Funktion, weichen Trennungen
//--- und Pdf-Stream-Machine fr Links-/Rechts-/Zentriert-
//--- und Blocksatz.
//--- Die C++-Version stoppt, wenn die vorgegebene Hhe mit
//--- Text vollgeschrieben ist.

#ifndef cWordLayout_hpp
#define cWordLayout_hpp

#include "cString.hpp"

#ifndef BYTE
#define BYTE unsigned char
#endif

class cWordLayout {
public:
	cWordLayout () { return; }

	static short ArialWidths [];
	static short ArialBoldWidths [];
	static short ArialItalic [];
	static short ArialBoldItalic [];
	static short TimesNewRomanWidths [];
	static short TimesNewRomanBoldWidths [];
	static short TimesNewRomanItalicWidths [];
	static short TimesNewRomanBoldItalicWidths [];
	static short CourierNewWidths [];

	static double Width (const char *str, short widths [])
	{
		if (widths == 0)
			widths = ArialWidths;

		double w = 0;
		int i, len = strlen (str);
		for (i = 0; i < len; i++)
			w += Width ((BYTE)str [i], widths);
		return w;
	}

	static double Width (int ch, short widths [])
	{
		if (widths == 0)
			widths = ArialWidths;

		if ((ch >= 32) && (ch <= 255))
			return widths [ch - 32];
		return widths [0];
		/*switch (ch) {		 //	UNICODE-Umsetzung
		case 8364: return widths [128 - 32];	// 
		case 8218: return widths [130 - 32];	// 
		case  402: return widths [131 - 32];	// 
		case 8222: return widths [132 - 32];	// 
		case 8230: return widths [133 - 32];	// 
		case 8224: return widths [134 - 32];	// 
		case 8225: return widths [135 - 32];	// 
		case  710: return widths [136 - 32];	// 
		case 8240: return widths [137 - 32];	// 
		case  352: return widths [138 - 32];	// 
		case 8249: return widths [139 - 32];	// 
		case  338: return widths [140 - 32];	// 
		case  381: return widths [142 - 32];	// 
		case 8216: return widths [145 - 32];	// 
		case 8217: return widths [146 - 32];	// 
		case 8220: return widths [147 - 32];	// 
		case 8221: return widths [148 - 32];	// 
		case 8226: return widths [149 - 32];	// 
		case 8211: return widths [150 - 32];	// 
		case 8212: return widths [151 - 32];	// 
		case  732: return widths [152 - 32];	// 
		case 8482: return widths [153 - 32];	// 
		case  353: return widths [154 - 32];	// 
		case 8250: return widths [155 - 32];	// 
		case  339: return widths [156 - 32];	// 
		case  382: return widths [158 - 32];	// 
		case  376: return widths [159 - 32];	// 
		}*/
	}

	static cAString holeWort (const char *string)
	{
		const char *nowBreaker = "-)]}";
		const char *nextBreaker = "([{";
		cAString wort = "";
		int len = strlen (string);
		for (int i = 0; i < len; i++) {
			if ((BYTE)string [i] < ' ') {
				return wort;
			} else if (string [i] == ' ') {
				wort += ' ';
				while (string [++i] == ' ')
					wort += ' ';
				return wort;
			} else if (strchr (nowBreaker, string [i])) {
				wort += string [i];
				while (string [++i] == ' ')		// 5.5.2009
					wort += ' ';
				return wort;
			} else if ((BYTE)string [i] == 173) {	// Bedingte Trennung
				wort += string [i];
				return wort;
			} else if ((i + 1 < len)			// 22.3.2009
				&& (strchr (nextBreaker, string [i + 1])) ) {
				wort += string [i];
				return wort;			// 5.5.2009
			} else
				wort += string [i];
		}
		return wort;
	}

	// Steuerzeichen wie tab, \r, \n drfen hier nicht drin sein.
	// Rckgabe: sind noch weitere Wrter da?
	struct layoutStruct_  {
		char *text;				//	[0] = die Zeichenkette, wird mit jedem Aufruf kleiner
		double size;			//	[1] = Schriftgre (wFac = 12 / 1000)
		double thisLineWidth;//	[2] = Zeilenbreite in Punkt (pt)
		short *charWidths;	//	[3]= Zeichenbreiten (Widths)
	};
	// returnArray:
	//		retArray [0] = gesamtlnge
	//		retArray [1] = wLen
	//		retArray [2] = Wort1
	//		retArray [3] = wLen
	//		retArray [4] = Wort2
	//		...
	static void lineLayout (layoutStruct_ &layoutStruct, const char *options, double &wordsDx, tCollection <double> &WoLen, cAStringArray &Worte)
	{
		cAString string = layoutStruct.text;
		short *widths = layoutStruct.charWidths;
		//while (widths.lenght < 256 - 32)	// widths bis 256 auffllen
		//	widths.push (widths [0]);
		double widthFac = layoutStruct.size / 1000;
		double breite = layoutStruct.thisLineWidth;
		int removeLastSpaces = 0;
		int takeMinChars = 2;	// Nur Worte und regulre Silben
		if (!options)
			options = "";
		if (strstr (options, "removeLastSpaces"))	// wichtig fr Zentrieren, Blocksatz und rechtbndig
			removeLastSpaces = 1;
		if (strstr (options, "takeMinChars0"))	//Wenn die Breite nicht fr das nchste Wort reicht
			takeMinChars = 0;
		else if (strstr (options, "takeMinChars1"))
			takeMinChars = 1;
		wordsDx = 0;	// die Gesamtlnge
		//Worte		// Silben vom vorigen Aufruf beachten
		//WoLen
		double lineLen = 0;
		int ende = 0;

		while (!ende) {
			cAString wort = holeWort (string);	//	Report ("'" + wort + "' ");
			if (!wort.Length ()) {	// wahrscheinlich ein Zeichen < ' '
				string = "";
				ende = 1;
			} else {
				double lenHard = 0;
				double lenSpace = 0;
				int len = wort.Length ();
				for (int i = 0; (i < len) && (wort [i] != ' '); i++)
					lenHard += Width (wort.CharAt (i), widths) * widthFac;
				for ( ; (i < len) && (wort [i] == ' '); i++)
					lenSpace += widths [0] * widthFac;

				if (lineLen + lenHard <= breite) {
					if (Worte.GetCount () > 0) {	// evtl. ein vorhergehendes 173 lschen und zusammenfgen
						cAString lwort = Worte.GetString (Worte.GetCount () - 1);
						if (lwort.LastChar () == 173) {	
							//--- Wegen eventuellem Blocksatz die Teile zusammenfgen
							lwort.SetAt (lwort.Length () - 1, 0);
							string.InsertString (0, lwort);
							wort  .InsertString (0, lwort);
							double recLen = WoLen.GetIt (WoLen.GetCount () - 1);	// 3.5.2012
							lineLen -= recLen;
							lenHard += recLen - widths [173 - 32] * widthFac;
							Worte.RemoveAndDelete (Worte.GetCount () - 1);
							WoLen.RemoveLast ();
						} else if ((strchr ("-)]}", lwort.CharAt (lwort.Length () - 1))
								||   (strchr ("([{", wort.CharAt (0))) && (lwort.LastChar () != ' '))) {
							//--- optionale Trennung wg Breakers rckgngig machen
							//--- Wegen eventuellem Blocksatz die Teile zusammenfgen
							string.InsertString (0, lwort);
							wort  .InsertString (0, lwort);
							lenHard += WoLen.GetIt (WoLen.GetCount () - 1);
							Worte.RemoveAndDelete (Worte.GetCount () - 1);
							WoLen.RemoveLast ();
						}
					}
					WoLen.Add (lenHard + lenSpace);
					Worte.AddString (wort);
					//Report (" push = " + wort);
					string.CutChars (0, wort.Length ());
					lineLen += lenHard + lenSpace;
					if (lineLen >= breite)
						ende = 1;
				} else if (!Worte.GetCount () && takeMinChars) {	// wenigstens ein Zeichen nehmen
					do {
						wort += string.CharAt (0);
						lineLen += Width (string.CharAt (0), widths) * widthFac;
						string.CutChars (0, 1);
					} while ((takeMinChars == 2)
						&& string.CharAt (0)
						&& (lineLen + Width (string.CharAt (0), widths) * widthFac <= breite));
					WoLen.Add (lineLen);
					Worte.AddString (wort);
					string.CutChars (0, wort.Length ());
					ende = 1;
				} else {
					ende = 1;
				}
			}
		}
		if (Worte.GetCount ()) {	// evtl. ein 173 konvertieren
			char *wort = (char *)Worte.GetString (Worte.GetCount () - 1);
			if (wort [strlen (wort) - 1] == 173) {
				wort [strlen (wort) - 1] = '-';	// ein normaler Bindestrich
				//rAr [rAr.length - 1] = wort;
				*WoLen.GetItAsPtr (WoLen.GetCount () - 1)
						= WoLen.GetIt (WoLen.GetCount () - 1)
							- widths [173 - 32] * widthFac
							+ widths [45 - 32] * widthFac;
			} else if (removeLastSpaces) {	// Leerzeichen am Ende kappen
				while (*wort && (wort [strlen (wort) - 1] == ' ')) {
					wort [strlen (wort) - 1] = 0;
					*WoLen.GetItAsPtr (WoLen.GetCount () - 1)
							= WoLen.GetIt (WoLen.GetCount () - 1)
									- widths [32 - 32] * widthFac;
				}
			}
			for (int i = 0; i < (int)WoLen.GetCount (); i++)
				wordsDx += WoLen.GetIt (i);
		}
		strcpy (layoutStruct.text, string);
	}

	//--- liefert Pdf-Stream:
	//--- 1 0 0 1 x y Tm 
	//--- h TL ...
	static cAString layoutAbsatz (const char *text, double *rectAr, double fontSize, short *widths, double lineHeight, int lcbr, int *done = 0)
	{
		cAString s, Text;
		int line = 0;
		if (!text)
			text = "";
		Text = text;
		{//--- alle Steuerzeichen < ' ' zu Leerzeichen machen
			BYTE *p = (BYTE *)Text.Get ();
			int i, max = Text.Length ();
			for (i = 0; i < max; i++)
				if (p [i] < ' ')
					p [i] = ' ';
		}
		//rectAr     = ifUndefined (rectAr, new Array ());
		if (fontSize < 1)
			fontSize = 1;
		if (!widths)
			widths = ArialWidths;
		if (lineHeight < 1)
			lineHeight = 1; // round (fontSize * 1.2, 1);
		if ((lcbr < 0) || (lcbr > 3))
			lcbr = 0;	// 0 = links, 1 = zentriert, 2 = block, 3 = rechts
		if (done)
			*done = 0;
		if (!*text || (rectAr [2] < 1))	// rectAr[0]= links, [1]=oben, [2]=breit, [3]=hoch
			return s;

		layoutStruct_ layoutStruct;
		layoutStruct.text = (char *)Text.Get ();
		layoutStruct.size = fontSize;
		layoutStruct.thisLineWidth = rectAr [2];	// x, y, dx, dy
		layoutStruct.charWidths = widths;

		s.Format ("1 0 0 1 %s %s Tm%s"
					 "%s TL%s"
						 , roundmax (rectAr [0], 2), roundmax (rectAr [1] - fontSize, 2), NL
						 , roundmax (lineHeight, 2), NL);

		double hoch = 0;		// es wird mindestens eine Zeile ausgegeben, auch wenn lineHeight > rectAr[3] ist
		if (done)				// hoch-Rechnung: hoch = fontSize + lineHeight + lineHeight ...
			hoch = fontSize;	// nach jedem Durchgang wird lineHeight addiert und mit rectAr[3] verglichen
		double lx = 0;

		do {
			//int len = strlen (layoutStruct.text);
			double wordsDx;
			tCollection <double> wLen;
			cAStringArray words;
			lineLayout (layoutStruct, lcbr ? "removeLastSpaces" : "", wordsDx, wLen, words);
			if (!wordsDx) {
				if (done)
					*done = strlen (text) - Text.Length ();
				cAString temp;
				temp.Set (layoutStruct.text, 50);
				temp.InsertString (0, "Abbruch in layoutAbsatz () bei: "); 
				alert (temp.Get ());
				return s;
			}

			double breite = rectAr [2];
			int i;
			double x;

			line++;	// wird zu nichts benutzt
			if (done)
				hoch += lineHeight;

			if (lcbr == 0) {	// linksbndig
				for (i = 0; i < (int)words.GetCount (); i++)
					s.FormatAdd ("(%s) Tj%s", EscapePsText (words.GetString (i)), NL);
			} else if (lcbr == 1) {	// zentriert
				x = (breite - wordsDx) / 2;
				if (lx != x)
					s.FormatAdd ("%s 0 Td%s", roundmax (x - lx, 2), NL);
				for (i = 0; i < (int)words.GetCount (); i++)
					s.FormatAdd ("(%s) Tj%s", EscapePsText (words.GetString (i)), NL);
				lx = x;
			} else if (lcbr == 2) {	// Blocksatz
				int addx = 0;
				if (words.GetCount () > 1)
					addx = -(int)floor ((breite - wordsDx) / (words.GetCount () - 1) * (1000.0 / fontSize));
				if ((addx == 0) || !strlen (layoutStruct.text) /*|| hoch > rectAr [3]*/) {	// letzte Zeile linksbndig
					for (i = 0; i < (int)words.GetCount (); i++)
						s.FormatAdd ("(%s) Tj%s", EscapePsText (words.GetString (i)), NL);
				} else {
					s.FormatAdd ("[(%s) ", EscapePsText (words.GetString (0)));
					for (i = 1; i < (int)words.GetCount (); i++)
						s.FormatAdd ("%d (%s) ", addx, EscapePsText (words.GetString (i)));
					s.FormatAdd ("] TJ%s", NL);
				}
			} else if (lcbr == 3) {	// rechtsbndig
				x = breite - wordsDx;
				if (lx != x)
					s.FormatAdd ("%s 0 Td%s", roundmax (x - lx, 2), NL);
				for (i = 0; i < (int)words.GetCount (); i++)
					s.FormatAdd ("(%s) Tj%s", EscapePsText (words.GetString (i)), NL);
				lx = x;
			}
			if (*layoutStruct.text && (!done || (hoch <= rectAr [3])))
				s.FormatAdd ("T*%s", NL);
		} while (*layoutStruct.text && (!done || (hoch <= rectAr [3])));
		if (done)
			*done = strlen (text) - Text.Length ();
		return s;
	}
};

short cWordLayout :: ArialWidths [] =
	{278,278,355,556,556,889,667,191,333,333,389,584,278,333,278,278	// sp - /
	,556,556,556,556,556,556,556,556,556,556,278,278,584,584,584,556	// 0 - ?
	,1015,667,667,722,722,667,611,778,722,278,500,667,556,833,722,778	// @ - O
	,667,778,722,667,611,722,667,944,667,667,611,278,278,278,469,556	// P - _
	,333,556,556,500,556,556,278,556,556,222,222,500,222,833,556,556	// ' - o
	,556,556,333,500,278,556,500,722,500,500,500,334,260,334,584,750	// p - del
	,556,750,222,556,333,1000,556,556,333,1000,667,333,1000,750,611,750	//  80  81  82  83  84  85  86  87  88  89  8a  8b  8c  8d  8e  8f
	,750,222,222,333,333,350,556,1000,333,1000,500,333,944,750,500,667	//  90  91  92  93  94  95  96  97  98  99  9a  9b  9c  9d  9e  9f
	,278,333,556,556,556,556,260,556,333,737,370,556,584,333,737,552	// 
	,400,549,333,333,333,576,537,278,333,333,365,556,834,834,834,611	// 
	,667,667,667,667,667,667,1000,722,667,667,667,667,278,278,278,278	// 
	,722,722,778,778,778,778,778,584,778,722,722,722,722,667,667,611	// 
	,556,556,556,556,556,556,889,500,556,556,556,556,278,278,278,278	// 
	,556,556,556,556,556,556,556,549,611,556,556,556,556,500,556,500};// 


short cWordLayout :: ArialBoldWidths [] =
	{278,333,474,556,556,889,722,238,333,333,389,584,278,333,278,278
	,556,556,556,556,556,556,556,556,556,556,333,333,584,584,584,611
	,975,722,722,722,722,667,611,778,722,278,556,722,611,833,722,778
	,667,778,722,667,611,722,667,944,667,667,611,333,278,333,584,556
	,333,556,611,556,611,556,333,611,611,278,278,556,278,889,611,611
	,611,611,389,556,333,611,556,778,556,556,500,389,280,389,584,750
	,556,750,278,556,500,1000,556,556,333,1000,667,333,1000,750,611,750
	,750,278,278,500,500,350,556,1000,333,1000,556,333,944,750,500,667
	,278,333,556,556,556,556,280,556,333,737,370,556,584,333,737,552
	,400,549,333,333,333,576,556,278,333,333,365,556,834,834,834,611
	,722,722,722,722,722,722,1000,722,667,667,667,667,278,278,278,278
	,722,722,778,778,778,778,778,584,778,722,722,722,722,667,667,611
	,556,556,556,556,556,556,889,556,556,556,556,556,278,278,278,278
	,611,611,611,611,611,611,611,549,611,611,611,611,611,556,611,556};

short cWordLayout :: ArialItalic [] =
	{278,278,355,556,556,889,667,191,333,333,389,584,278,333,278,278
	,556,556,556,556,556,556,556,556,556,556,278,278,584,584,584,556
	,1015,667,667,722,722,667,611,778,722,278,500,667,556,833,722,778
	,667,778,722,667,611,722,667,944,667,667,611,278,278,278,469,556
	,333,556,556,500,556,556,278,556,556,222,222,500,222,833,556,556
	,556,556,333,500,278,556,500,722,500,500,500,334,260,334,584,750
	,556,750,222,556,333,1000,556,556,333,1000,667,333,1000,750,611,750
	,750,222,222,333,333,350,556,1000,333,1000,500,333,944,750,500,667
	,278,333,556,556,556,556,260,556,333,737,370,556,584,333,737,552
	,400,549,333,333,333,576,537,278,333,333,365,556,834,834,834,611
	,667,667,667,667,667,667,1000,722,667,667,667,667,278,278,278,278
	,722,722,778,778,778,778,778,584,778,722,722,722,722,667,667,611
	,556,556,556,556,556,556,889,500,556,556,556,556,278,278,278,278
	,556,556,556,556,556,556,556,549,611,556,556,556,556,500,556,500};

short cWordLayout :: ArialBoldItalic [] =
	{278,333,474,556,556,889,722,238,333,333,389,584,278,333,278,278
	,556,556,556,556,556,556,556,556,556,556,333,333,584,584,584,611
	,975,722,722,722,722,667,611,778,722,278,556,722,611,833,722,778
	,667,778,722,667,611,722,667,944,667,667,611,333,278,333,584,556
	,333,556,611,556,611,556,333,611,611,278,278,556,278,889,611,611
	,611,611,389,556,333,611,556,778,556,556,500,389,280,389,584,750
	,556,750,278,556,500,1000,556,556,333,1000,667,333,1000,750,611,750
	,750,278,278,500,500,350,556,1000,333,1000,556,333,944,750,500,667
	,278,333,556,556,556,556,280,556,333,737,370,556,584,333,737,552
	,400,549,333,333,333,576,556,278,333,333,365,556,834,834,834,611
	,722,722,722,722,722,722,1000,722,667,667,667,667,278,278,278,278
	,722,722,778,778,778,778,778,584,778,722,722,722,722,667,667,611
	,556,556,556,556,556,556,889,556,556,556,556,556,278,278,278,278
	,611,611,611,611,611,611,611,549,611,611,611,611,611,556,611,556};

short cWordLayout :: TimesNewRomanWidths [] =
	{250,333,408,500,500,833,778,180,333,333,500,564,250,333,250,278
	,500,500,500,500,500,500,500,500,500,500,278,278,564,564,564,444
	,921,722,667,667,722,611,556,722,722,333,389,722,611,889,722,722
	,556,722,667,556,611,722,722,944,722,722,611,333,278,333,469,500
	,333,444,500,444,500,444,333,500,500,278,278,500,278,778,500,500
	,500,500,333,389,278,500,500,722,500,500,444,480,200,480,541,778
	,500,778,333,500,444,1000,500,500,333,1000,556,333,889,778,611,778
	,778,333,333,444,444,350,500,1000,333,980,389,333,722,778,444,722
	,250,333,500,500,500,500,200,500,333,760,276,500,564,333,760,500
	,400,549,300,300,333,576,453,250,333,300,310,500,750,750,750,444
	,722,722,722,722,722,722,889,667,611,611,611,611,333,333,333,333
	,722,722,722,722,722,722,722,564,722,722,722,722,722,722,556,500
	,444,444,444,444,444,444,667,444,444,444,444,444,278,278,278,278
	,500,500,500,500,500,500,500,549,500,500,500,500,500,500,500,500};

short cWordLayout :: TimesNewRomanBoldWidths [] =
	{250,333,555,500,500,1000,833,278,333,333,500,570,250,333,250,278
	,500,500,500,500,500,500,500,500,500,500,333,333,570,570,570,500
	,930,722,667,722,722,667,611,778,778,389,500,778,667,944,722,778
	,611,778,722,556,667,722,722,1000,722,722,667,333,278,333,581,500
	,333,500,556,444,556,444,333,500,556,278,333,556,278,833,556,500
	,556,556,444,389,333,556,500,722,500,500,444,394,220,394,520,778
	,500,778,333,500,500,1000,500,500,333,1000,556,333,1000,778,667,778
	,778,333,333,500,500,350,500,1000,333,1000,389,333,722,778,444,722
	,250,333,500,500,500,500,220,500,333,747,300,500,570,333,747,500
	,400,549,300,300,333,576,540,250,333,300,330,500,750,750,750,500
	,722,722,722,722,722,722,1000,722,667,667,667,667,389,389,389,389
	,722,722,778,778,778,778,778,570,778,722,722,722,722,722,611,556
	,500,500,500,500,500,500,722,444,444,444,444,444,278,278,278,278
	,500,556,500,500,500,500,500,549,500,556,556,556,556,500,556,500};

short cWordLayout :: TimesNewRomanItalicWidths [] =
	{250,333,420,500,500,833,778,214,333,333,500,675,250,333,250,278
	,500,500,500,500,500,500,500,500,500,500,333,333,675,675,675,500
	,920,611,611,667,722,611,611,722,722,333,444,667,556,833,667,722
	,611,722,611,500,556,722,611,833,611,556,556,389,278,389,422,500
	,333,500,500,444,500,444,278,500,500,278,278,444,278,722,500,500
	,500,500,389,389,278,500,444,667,444,444,389,400,275,400,541,778
	,500,778,333,500,556,889,500,500,333,1000,500,333,944,778,556,778
	,778,333,333,556,556,350,500,889,333,980,389,333,667,778,389,556
	,250,389,500,500,500,500,275,500,333,760,276,500,675,333,760,500
	,400,549,300,300,333,576,523,250,333,300,310,500,750,750,750,500
	,611,611,611,611,611,611,889,667,611,611,611,611,333,333,333,333
	,722,667,722,722,722,722,722,675,722,722,722,722,722,556,611,500
	,500,500,500,500,500,500,667,444,444,444,444,444,278,278,278,278
	,500,500,500,500,500,500,500,549,500,500,500,500,500,444,500,444};

short cWordLayout :: TimesNewRomanBoldItalicWidths [] =
	{250,389,555,500,500,833,778,278,333,333,500,570,250,333,250,278
	,500,500,500,500,500,500,500,500,500,500,333,333,570,570,570,500
	,832,667,667,667,722,667,667,722,778,389,500,667,611,889,722,722
	,611,722,667,556,611,722,667,889,667,611,611,333,278,333,570,500
	,333,500,500,444,500,444,333,500,556,278,278,500,278,778,556,500
	,500,500,389,389,278,556,444,667,500,444,389,348,220,348,570,778
	,500,778,333,500,500,1000,500,500,333,1000,556,333,944,778,611,778
	,778,333,333,500,500,350,500,1000,333,1000,389,333,722,778,389,611
	,250,389,500,500,500,500,220,500,333,747,266,500,606,333,747,500
	,400,549,300,300,333,576,500,250,333,300,300,500,750,750,750,500
	,667,667,667,667,667,667,944,667,667,667,667,667,389,389,389,389
	,722,722,722,722,722,722,722,570,722,722,722,722,722,611,611,500
	,500,500,500,500,500,500,722,444,444,444,444,444,278,278,278,278
	,500,556,500,500,500,500,500,549,500,556,556,556,556,444,500,444};

short cWordLayout :: CourierNewWidths [] =
	{600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600
	,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600,600};

#endif
